@@ -1,15 +1,12 @@
 /* Frei0r wrapper for Weed
    author: Salsaman (G. Finch) <salsaman@gmail.com>
 
-   Released under the Lesser Gnu Public License (LGPL) 3 or later
+   Released under the Gnu Public License (GPL) 3 or later
    See www.gnu.org for details
 
    (c) 2005 - 2012, Salsaman
 */
 
-
-
-
 #include <frei0r.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,8 +44,9 @@ static int package_version = 1; // version of this package
 #endif
 
 #include "weed-utils-code.c" // optional
-#include "weed-plugin-utils.c" // optional
 
+#define NEED_ALPHA_SORT
+#include "weed-plugin-utils.c" // optional
 
 /////////////////////////////////////////////////////////////
 typedef f0r_instance_t (*f0r_construct_f)(unsigned int width, unsigned int height);
@@ -100,10 +98,9 @@ static int getenv_piece(char *target, size_t tlen, char *envvar, int num) {
   return 1;
 }
 
+
 ////////////////////////////////////////////////////////////////
 
-
-
 int frei0r_init(weed_plant_t *inst) {
   weed_plant_t *out_channel, *filter;
   int error, height, width, cpalette;
@@ -120,7 +117,6 @@ int frei0r_init(weed_plant_t *inst) {
   if (cpalette == WEED_PALETTE_UYVY || cpalette == WEED_PALETTE_YUYV) width >>= 1;
   else width >>= 2;
 
-
   f0r_construct = weed_get_voidptr_value(filter, "plugin_f0r_construct", &error);
 
   if ((f0r_inst = (*f0r_construct)(width, height)) == NULL) return WEED_ERROR_INIT_ERROR;
@@ -199,10 +195,6 @@ static void weed_params_to_frei0r_params(weed_plant_t *inst, weed_plant_t **in_p
 }
 
 
-
-
-
-
 int frei0r_process(weed_plant_t *inst, weed_timecode_t timestamp) {
   int error;
 
@@ -265,7 +257,6 @@ int frei0r_process(weed_plant_t *inst, weed_timecode_t timestamp) {
 }
 
 
-
 weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
   weed_plant_t *plugin_info;
   if (FREI0R_MAJOR_VERSION < 1 || FREI0R_MINOR_VERSION < 1) return NULL;
@@ -273,8 +264,9 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
   plugin_info = weed_plugin_info_init(weed_boot, num_versions, api_versions);
 
   if (plugin_info != NULL) {
+    dlink_list_t *list = NULL;
     int *pal;
-    int pnum, wnum, num_weed_params;
+
     char **rfx_strings = NULL;
 
     char *blacklist[2] = {"Timeout indicator", NULL};
@@ -288,6 +280,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
     int num_filters = 0;
 
     int finished = 0;
+    int pnum, wnum, num_weed_params;
 
     char vdir1[PATH_MAX] = "/usr/lib/frei0r-1/";
     char vdir2[PATH_MAX] = "/usr/local/lib/frei0r-1/";
@@ -358,6 +351,9 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
 
     while (vdirval < 6 || vdirval > 9) {
       // step through each of our frei0r dirs
+      if (curvdir != NULL) closedir(curvdir);
+      curvdir = NULL;
+
       if (vdirval == 0) {
         curvdir = opendir(vdir3);
         if (curvdir == NULL) vdirval = 2;
@@ -365,7 +361,6 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
       }
 
       if (vdirval == 2) {
-        if (curvdir != NULL) closedir(curvdir);
         curvdir = opendir(vdir2);
         if (curvdir == NULL) vdirval = 4;
         else {
@@ -374,7 +369,6 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
       }
 
       if (vdirval == 4) {
-        if (curvdir != NULL) closedir(curvdir);
         curvdir = opendir(vdir1);
         if (curvdir == NULL) {
           vdirval = 6;
@@ -383,7 +377,6 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
         vdirval = 5;
       }
 
-
       if (vdirval > 9) {
         char *fpp_copy = strdup(fpp);
         if (!getenv_piece(vdir1, PATH_MAX, fpp_copy, vdirval - 10)) {
@@ -408,6 +401,9 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
       weed_memset(vendor_name, 0, 1);
 
       do {
+        if (curdir != NULL) closedir(curdir);
+        curdir = NULL;
+
         snprintf(dir1, PATH_MAX, "%s/%s", vdir1, vendor_name);
 
         if (vdirval < 10) {
@@ -429,14 +425,12 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
             continue;
           }
         } else if (vdirval == 3) {
-          if (curdir != NULL) closedir(curdir);
           curdir = opendir(dir2);
           if (curdir == NULL) {
             if (vdirent == NULL) break;
             continue;
           }
         } else if (vdirval == 5 || vdirval > 9) {
-          if (curdir != NULL) closedir(curdir);
           curdir = opendir(dir1);
           if (curdir == NULL) {
             if (vdirent == NULL) break;
@@ -462,12 +456,16 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
           snprintf(plug1, PATH_MAX, "%s/%s", dir1, plugin_name);
 
           if (vdirval < 10) {
+            if (handle != NULL) dlclose(handle);
+            handle = NULL;
+
             snprintf(plug2, PATH_MAX, "%s/%s", dir2, plugin_name);
             snprintf(plug3, PATH_MAX, "%s/%s", dir3, plugin_name);
 
             handle = dlopen(plug3, RTLD_NOW);
             if ((handle != NULL && (vdirval > 1)) || (handle == NULL && vdirval == 1)) {
               if (handle != NULL) dlclose(handle);
+              handle = NULL;
               continue;
             }
 
@@ -475,6 +473,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
               handle = dlopen(plug2, RTLD_NOW);
               if ((handle != NULL && (vdirval > 3)) || (handle == NULL && vdirval == 3)) {
                 if (handle != NULL) dlclose(handle);
+                handle = NULL;
                 continue;
               }
             }
@@ -487,31 +486,37 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
 
           if ((f0r_deinit = dlsym(handle, "f0r_deinit")) == NULL) {
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
           if ((f0r_init = dlsym(handle, "f0r_init")) == NULL) {
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
           if ((f0r_get_plugin_info = dlsym(handle, "f0r_get_plugin_info")) == NULL) {
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
           if ((f0r_get_param_info = dlsym(handle, "f0r_get_param_info")) == NULL) {
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
           if ((f0r_construct = dlsym(handle, "f0r_construct")) == NULL) {
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
           if ((f0r_destruct = dlsym(handle, "f0r_destruct")) == NULL) {
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
@@ -533,6 +538,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
           if (blacklisted || f0rinfo.frei0r_version != FREI0R_MAJOR_VERSION) {
             (*f0r_deinit)();
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
@@ -542,6 +548,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
             if ((f0r_update = dlsym(handle, "f0r_update")) == NULL) {
               (*f0r_deinit)();
               dlclose(handle);
+              handle = NULL;
               continue;
             }
             break;
@@ -550,12 +557,14 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
             if ((f0r_update2 = dlsym(handle, "f0r_update2")) == NULL) {
               (*f0r_deinit)();
               dlclose(handle);
+              handle = NULL;
               continue;
             }
             break;
           default:
             (*f0r_deinit)();
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
@@ -579,6 +588,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
           } else {
             f0r_deinit();
             dlclose(handle);
+            handle = NULL;
             continue;
           }
 
@@ -649,6 +659,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
           default:
             (*f0r_deinit)();
             dlclose(handle);
+            handle = NULL;
             weed_free(pal);
             weed_free(out_chantmpls);
             if (in_chantmpls != NULL) weed_free(in_chantmpls);
@@ -663,6 +674,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
           if ((f0r_inst = (*f0r_construct)(640, 480)) == NULL) {
             (*f0r_deinit)();
             dlclose(handle);
+            handle = NULL;
             weed_free(out_chantmpls);
             if (in_chantmpls != NULL) weed_free(in_chantmpls);
             continue;
@@ -676,6 +688,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
 #endif
               (*f0r_deinit)();
               dlclose(handle);
+              handle = NULL;
               weed_free(out_chantmpls);
               if (in_chantmpls != NULL) weed_free(in_chantmpls);
               continue;
@@ -686,6 +699,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
               f0r_destruct(f0r_inst);
               (*f0r_deinit)();
               dlclose(handle);
+              handle = NULL;
               weed_free(out_chantmpls);
               if (in_chantmpls != NULL) weed_free(in_chantmpls);
               continue;
@@ -836,6 +850,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
 #endif
                 (*f0r_deinit)();
                 dlclose(handle);
+                handle = NULL;
                 weed_free(out_chantmpls);
                 if (in_chantmpls != NULL) weed_free(in_chantmpls);
                 if (in_params != NULL) weed_free(in_params);
@@ -901,7 +916,8 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
             break;
           }
 
-          weed_plugin_info_add_filter_class(plugin_info, filter_class);
+          list = add_to_list_sorted(list, filter_class, f0rinfo.name);
+
           // end plugin
         }
         // end vendor dir
@@ -910,9 +926,18 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
       curvdir = NULL;
       vdirval++;
       if (curdir != NULL) closedir(curdir);
+      curdir = NULL;
     }
     // end frei0r dirs
 
+    if (curvdir != NULL) closedir(curvdir);
+    curvdir = NULL;
+    if (curdir != NULL) closedir(curdir);
+    curdir = NULL;
+
+    if (handle != NULL) dlclose(handle);
+    handle = NULL;
+
 #ifndef DEBUG
     dup2(new_stdout, 1);
     dup2(new_stderr, 2);
@@ -922,6 +947,9 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
       fprintf(stderr, "No frei0r plugins found; if you have them installed please set the FREI0R_PATH environment variable to point to them.\n");
       return NULL;
     }
+
+    add_filters_from_list(plugin_info, list);
+
     weed_set_int_value(plugin_info, "version", package_version);
   }
 
