@@ -1,6 +1,6 @@
 // framedraw.c
 // LiVES
-// (c) G. Finch (salsaman+lives@gmail.com) 2002 - 2018
+// (c) G. Finch (salsaman+lives@gmail.com) 2002 - 2019
 // see file COPYING for licensing details : released under the GNU GPL 3 or later
 
 // functions for the 'framedraw' widget - lets users draw on frames :-)
@@ -40,7 +40,7 @@ static double calc_fd_scale(int width, int height) {
   }
 
   if (width > MAX_PRE_X) scale = (double)width / (double)MAX_PRE_X;
-  if (height > MAX_PRE_Y && (height / MAX_PRE_Y > scale)) scale = (double)height / (double)MAX_PRE_Y;
+  if (height > MAX_PRE_Y && (height / MAX_PRE_Y < scale)) scale = (double)height / (double)MAX_PRE_Y;
   return scale;
 }
 
@@ -89,12 +89,35 @@ static void start_preview(LiVESButton *button, lives_rfx_t *rfx) {
 }
 
 
+static void framedraw_redraw_cb(LiVESWidget *widget, lives_special_framedraw_rect_t *framedraw) {
+  framedraw_redraw(framedraw, FALSE, NULL);
+}
+
+
+static void after_framedraw_frame_spinbutton_changed(LiVESSpinButton *spinbutton, lives_special_framedraw_rect_t *framedraw) {
+  // update the single frame/framedraw preview
+  // after the "frame number" spinbutton has changed
+  mainw->framedraw_frame = lives_spin_button_get_value_as_int(spinbutton);
+  if (lives_widget_is_visible(mainw->framedraw_preview)) {
+    if (mainw->framedraw_preview != NULL) lives_widget_set_sensitive(mainw->framedraw_preview, FALSE);
+    lives_widget_context_update();
+    load_rfx_preview(framedraw->rfx);
+  } else framedraw_redraw(framedraw, TRUE, NULL);
+}
+
+
 void framedraw_connect_spinbutton(lives_special_framedraw_rect_t *framedraw, lives_rfx_t *rfx) {
   framedraw->rfx = rfx;
 
   lives_signal_connect_after(LIVES_GUI_OBJECT(mainw->framedraw_spinbutton), LIVES_WIDGET_VALUE_CHANGED_SIGNAL,
                              LIVES_GUI_CALLBACK(after_framedraw_frame_spinbutton_changed),
                              framedraw);
+  lives_signal_connect_after(LIVES_GUI_OBJECT(mainw->framedraw_opscale), LIVES_WIDGET_VALUE_CHANGED_SIGNAL,
+                             LIVES_GUI_CALLBACK(framedraw_redraw_cb),
+                             framedraw);
+  lives_signal_connect(LIVES_GUI_OBJECT(mainw->framedraw_cbutton), LIVES_WIDGET_COLOR_SET_SIGNAL,
+                       LIVES_GUI_CALLBACK(framedraw_redraw_cb),
+                       framedraw);
 }
 
 
@@ -126,8 +149,10 @@ void framedraw_add_label(LiVESVBox *box) {
   LiVESWidget *label;
 
   // TRANSLATORS - Preview refers to preview window; keep this phrase short
+  widget_opts.justify = LIVES_JUSTIFY_CENTER;
   label = lives_standard_label_new(_("You can click in Preview to change these values"));
-  lives_box_pack_start(LIVES_BOX(box), label, FALSE, FALSE, 0);
+  lives_box_pack_start(LIVES_BOX(box), label, FALSE, FALSE, widget_opts.packing_height / 2.);
+  widget_opts.justify = LIVES_JUSTIFY_DEFAULT;
 }
 
 
@@ -138,7 +163,7 @@ void framedraw_add_reset(LiVESVBox *box, lives_special_framedraw_rect_t *framedr
 
   mainw->framedraw_reset = lives_standard_button_new_from_stock(LIVES_STOCK_REFRESH, NULL);
   hbox_rst = lives_hbox_new(FALSE, 0);
-  lives_box_pack_start(LIVES_BOX(box), hbox_rst, FALSE, FALSE, 0);
+  lives_box_pack_start(LIVES_BOX(box), hbox_rst, FALSE, FALSE, 4. * widget_opts.scale);
 
   lives_button_set_label(LIVES_BUTTON(mainw->framedraw_reset), _("_Reset Values"));
   lives_box_pack_start(LIVES_BOX(hbox_rst), mainw->framedraw_reset, TRUE, FALSE, 0);
@@ -159,13 +184,15 @@ void widget_add_framedraw(LiVESVBox *box, int start, int end, boolean add_previe
   // the redraw button should be connected to an appropriate redraw function
   // after calling this function
 
-  // an example of this is in 'trim frames'
   LiVESAdjustment *spinbutton_adj;
 
   LiVESWidget *vseparator;
   LiVESWidget *vbox;
   LiVESWidget *hbox;
+  LiVESWidget *label;
+  LiVESWidget *cbutton;
   LiVESWidget *frame;
+  lives_colRGBA64_t opcol;
 
   lives_rfx_t *rfx;
 
@@ -204,6 +231,7 @@ void widget_add_framedraw(LiVESVBox *box, int start, int end, boolean add_previe
   lives_container_add(LIVES_CONTAINER(frame), hbox);
 
   if (palette->style & STYLE_1) {
+    lives_widget_set_bg_color(frame, LIVES_WIDGET_STATE_NORMAL, &palette->normal_back);
     lives_widget_set_bg_color(hbox, LIVES_WIDGET_STATE_NORMAL, &palette->normal_back);
   }
 
@@ -211,6 +239,10 @@ void widget_add_framedraw(LiVESVBox *box, int start, int end, boolean add_previe
   lives_widget_set_size_request(mainw->framedraw, width, height);
   lives_container_set_border_width(LIVES_CONTAINER(mainw->framedraw), 1);
 
+  if (palette->style & STYLE_1) {
+    lives_widget_set_bg_color(hbox, LIVES_WIDGET_STATE_NORMAL, &palette->normal_fore);
+  }
+
   lives_widget_set_events(mainw->framedraw, LIVES_BUTTON1_MOTION_MASK | LIVES_BUTTON_RELEASE_MASK |
                           LIVES_BUTTON_PRESS_MASK | LIVES_ENTER_NOTIFY_MASK | LIVES_LEAVE_NOTIFY_MASK);
 
@@ -218,7 +250,6 @@ void widget_add_framedraw(LiVESVBox *box, int start, int end, boolean add_previe
 
   lives_box_pack_start(LIVES_BOX(hbox), fbord_eventbox, FALSE, FALSE, 0);
 
-  //lives_container_add(LIVES_CONTAINER(frame), fbord_eventbox);
   lives_container_add(LIVES_CONTAINER(fbord_eventbox), mainw->framedraw);
 
   if (palette->style & STYLE_1) {
@@ -229,6 +260,18 @@ void widget_add_framedraw(LiVESVBox *box, int start, int end, boolean add_previe
   lives_signal_connect_after(LIVES_GUI_OBJECT(mainw->framedraw), LIVES_WIDGET_EXPOSE_EVENT,
                              LIVES_GUI_CALLBACK(expose_fd_event), NULL);
 
+  // mask colour and opacity controls
+  mainw->framedraw_maskbox = lives_hbox_new(FALSE, 2);
+  lives_box_pack_start(LIVES_BOX(vbox), mainw->framedraw_maskbox, FALSE, FALSE, widget_opts.packing_height);
+  label = lives_standard_label_new(_("Mask: opacity"));
+  lives_box_pack_start(LIVES_BOX(mainw->framedraw_maskbox), label, FALSE, FALSE, widget_opts.packing_width);
+  spinbutton_adj = lives_adjustment_new(0.25, 0.0, 1.0, 0.1, 0.1, 0);
+  mainw->framedraw_opscale = lives_standard_hscale_new(LIVES_ADJUSTMENT(spinbutton_adj));
+  lives_box_pack_start(LIVES_BOX(mainw->framedraw_maskbox), mainw->framedraw_opscale, TRUE, TRUE, 0);
+  opcol = lives_rgba_col_new(0, 0, 0, 65535);
+  cbutton = lives_standard_color_button_new(LIVES_BOX(mainw->framedraw_maskbox), _("color"), FALSE, &opcol, NULL, NULL, NULL, NULL);
+  mainw->framedraw_cbutton = cbutton; // stop compiler complaining
+
   hbox = lives_hbox_new(FALSE, 2);
   lives_box_pack_start(LIVES_BOX(vbox), hbox, FALSE, FALSE, widget_opts.packing_height);
 
@@ -237,13 +280,16 @@ void widget_add_framedraw(LiVESVBox *box, int start, int end, boolean add_previe
 
   spinbutton_adj = lives_spin_button_get_adjustment(LIVES_SPIN_BUTTON(mainw->framedraw_spinbutton));
 
-  mainw->framedraw_scale = lives_hscale_new(LIVES_ADJUSTMENT(spinbutton_adj));
-  lives_box_pack_start(LIVES_BOX(hbox), mainw->framedraw_scale, TRUE, TRUE, 0);
-  lives_scale_set_draw_value(LIVES_SCALE(mainw->framedraw_scale), FALSE);
-
   rfx = (lives_rfx_t *)lives_widget_object_get_data(LIVES_WIDGET_OBJECT(lives_widget_get_toplevel(LIVES_WIDGET(box))), "rfx");
   mainw->framedraw_preview = lives_standard_button_new_from_stock(LIVES_STOCK_REFRESH, _("_Preview"));
   lives_box_pack_start(LIVES_BOX(hbox), mainw->framedraw_preview, TRUE, FALSE, 0);
+
+  hbox = lives_hbox_new(FALSE, 2);
+  lives_box_pack_start(LIVES_BOX(vbox), hbox, FALSE, FALSE, widget_opts.packing_height);
+
+  mainw->framedraw_scale = lives_standard_hscale_new(LIVES_ADJUSTMENT(spinbutton_adj));
+  lives_box_pack_start(LIVES_BOX(hbox), mainw->framedraw_scale, TRUE, TRUE, widget_opts.border_width);
+
   lives_widget_set_sensitive(mainw->framedraw_spinbutton, FALSE);
   lives_widget_set_sensitive(mainw->framedraw_scale, FALSE);
   lives_signal_connect(mainw->framedraw_preview, LIVES_WIDGET_CLICKED_SIGNAL, LIVES_GUI_CALLBACK(start_preview), rfx);
@@ -253,12 +299,13 @@ void widget_add_framedraw(LiVESVBox *box, int start, int end, boolean add_previe
   if (!add_preview_button) {
     lives_widget_hide(mainw->framedraw_preview);
   }
+
+  lives_widget_hide(mainw->framedraw_maskbox);
 }
 
 
 void framedraw_redraw(lives_special_framedraw_rect_t *framedraw, boolean reload, LiVESPixbuf *pixbuf) {
-  // this will draw the mask (framedraw_bitmap) and optionally reload the image
-  // and then combine them
+  // this will draw the overlay and optionally reload the image
 
   int fd_height;
   int fd_width;
@@ -276,6 +323,8 @@ void framedraw_redraw(lives_special_framedraw_rect_t *framedraw, boolean reload,
   fd_width = lives_widget_get_allocation_width(mainw->framedraw);
   fd_height = lives_widget_get_allocation_height(mainw->framedraw);
 
+  if (fd_width < 4 || fd_height < 4) return;
+
   width = cfile->hsize;
   height = cfile->vsize;
 
@@ -305,48 +354,8 @@ void framedraw_redraw(lives_special_framedraw_rect_t *framedraw, boolean reload,
   // her we dont offset because we are drawing in the pixbuf, not the widget
 
   switch (framedraw->type) {
-  case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTRECT: // deprecated
-    // scale values
-    if (framedraw->xstart_param->dp == 0) {
-      xstartf = (double)lives_spin_button_get_value_as_int(LIVES_SPIN_BUTTON(framedraw->xstart_param->widgets[0]));
-      xstartf = xstartf / (double)cfile->hsize * (double)width;
-    } else {
-      xstartf = lives_spin_button_get_value(LIVES_SPIN_BUTTON(framedraw->xstart_param->widgets[0]));
-      xstartf = xstartf * (double)width;
-    }
-
-    if (framedraw->xend_param->dp == 0) {
-      xendf = (double)lives_spin_button_get_value_as_int(LIVES_SPIN_BUTTON(framedraw->xend_param->widgets[0]));
-      xendf = xendf / (double)cfile->hsize * (double)width;
-    } else {
-      xendf = lives_spin_button_get_value(LIVES_SPIN_BUTTON(framedraw->xend_param->widgets[0]));
-      xendf = xendf * (double)width;
-    }
-
-    if (framedraw->ystart_param->dp == 0) {
-      ystartf = (double)lives_spin_button_get_value_as_int(LIVES_SPIN_BUTTON(framedraw->ystart_param->widgets[0]));
-      ystartf = ystartf / (double)cfile->vsize * (double)height;
-    } else {
-      ystartf = lives_spin_button_get_value(LIVES_SPIN_BUTTON(framedraw->ystart_param->widgets[0]));
-      ystartf = ystartf * (double)height;
-    }
-
-    if (framedraw->yend_param->dp == 0) {
-      yendf = lives_spin_button_get_value_as_int(LIVES_SPIN_BUTTON(framedraw->yend_param->widgets[0]));
-      yendf = yendf / (double)cfile->vsize * (double)height;
-    } else {
-      yendf = lives_spin_button_get_value(LIVES_SPIN_BUTTON(framedraw->yend_param->widgets[0]));
-      yendf = yendf * (double)height;
-    }
-
-    lives_painter_set_source_rgb(cr, 1., 0., 0.);
-    lives_painter_rectangle(cr, xstartf - 1., ystartf - 1., xendf + 2., yendf + 2.);
-    lives_painter_stroke(cr);
-
-    break;
   case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT:
   case LIVES_PARAM_SPECIAL_TYPE_RECT_DEMASK:
-
     if (framedraw->xstart_param->dp == 0) {
       xstartf = (double)lives_spin_button_get_value_as_int(LIVES_SPIN_BUTTON(framedraw->xstart_param->widgets[0]));
       xstartf = xstartf / (double)cfile->hsize * (double)width;
@@ -379,21 +388,25 @@ void framedraw_redraw(lives_special_framedraw_rect_t *framedraw, boolean reload,
       yendf = yendf * (double)height;
     }
 
-    if (b1_held || framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT) {
+    if (framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT) {
       lives_painter_set_source_rgb(cr, 1., 0., 0.);
-      lives_painter_rectangle(cr, xstartf - 1., ystartf - 1., xendf - xstartf + 2., yendf - ystartf + 2.);
+      lives_painter_rectangle(cr, xstartf - 1., ystartf - 1., xendf, yendf);
       lives_painter_stroke(cr);
     } else {
-      if (!b1_held) {
-        // create a mask which is only opaque within the clipping area
-
-        lives_painter_rectangle(cr, 0, 0, width, height);
-        lives_painter_rectangle(cr, xstartf, ystartf, xendf - xstartf + 1., yendf - ystartf + 1.);
-        lives_painter_set_operator(cr, LIVES_PAINTER_OPERATOR_DEST_OUT);
-        lives_painter_set_source_rgba(cr, .0, .0, .0, .5);
-        lives_painter_set_fill_rule(cr, LIVES_PAINTER_FILL_RULE_EVEN_ODD);
-        lives_painter_fill(cr);
+      if (b1_held) {
+        lives_painter_set_source_rgb(cr, 1., 0., 0.);
+        lives_painter_rectangle(cr, xstartf - 1., ystartf - 1., xendf - xstartf + 2., yendf - ystartf + 2.);
+        lives_painter_stroke(cr);
       }
+      // create a mask which is only opaque within the clipping area
+      LiVESWidgetColor maskcol;
+      double opacity = lives_range_get_value(LIVES_RANGE(mainw->framedraw_opscale));
+      lives_color_button_get_color(LIVES_COLOR_BUTTON(mainw->framedraw_cbutton), &maskcol);
+      lives_painter_rectangle(cr, 0, 0, width, height);
+      lives_painter_rectangle(cr, xstartf, ystartf, xendf - xstartf + 1., yendf - ystartf + 1.);
+      lives_painter_set_source_rgba(cr, maskcol.red, maskcol.green, maskcol.blue, opacity);
+      lives_painter_set_fill_rule(cr, LIVES_PAINTER_FILL_RULE_EVEN_ODD);
+      lives_painter_fill(cr);
     }
 
     break;
@@ -499,7 +512,6 @@ void load_rfx_preview(lives_rfx_t *rfx) {
         if (is_virtual_frame(mainw->current_file, vend)) {
           retb = virtual_to_images(mainw->current_file, vend, vend, FALSE, NULL);
           if (!retb) {
-            fclose(infofile);
             return;
           }
         }
@@ -520,7 +532,7 @@ void load_rfx_preview(lives_rfx_t *rfx) {
   do {
     retval = 0;
     mainw->read_failed = FALSE;
-    lives_fgets(mainw->msg, 512, infofile);
+    lives_fgets(mainw->msg, MAINW_MSG_SIZE, infofile);
     if (mainw->read_failed) retval = do_read_failed_error_s_with_retry(cfile->info_file, NULL, NULL);
   } while (retval == LIVES_RESPONSE_RETRY);
 
@@ -582,21 +594,11 @@ void load_rfx_preview(lives_rfx_t *rfx) {
 }
 
 
-void after_framedraw_frame_spinbutton_changed(LiVESSpinButton *spinbutton, lives_special_framedraw_rect_t *framedraw) {
-  // update the single frame/framedraw preview
-  // after the "frame number" spinbutton has changed
-  mainw->framedraw_frame = lives_spin_button_get_value_as_int(spinbutton);
-  if (lives_widget_is_visible(mainw->framedraw_preview)) {
-    if (mainw->framedraw_preview != NULL) lives_widget_set_sensitive(mainw->framedraw_preview, FALSE);
-    lives_widget_context_update();
-    load_rfx_preview(framedraw->rfx);
-  } else framedraw_redraw(framedraw, TRUE, NULL);
-}
-
-
 void load_framedraw_image(LiVESPixbuf *pixbuf) {
   // this is for the single frame framedraw widget
-  // it should be called whenever mainw->framedraw_bitmap changes
+  // it should be called whenever mainw->framedraw_frame changes
+  // the pixbuf will be loaded if it is NULL
+  // then it is converted to fd_layer_orig
 
   weed_timecode_t tc;
 
@@ -612,6 +614,10 @@ void load_framedraw_image(LiVESPixbuf *pixbuf) {
     pixbuf = pull_lives_pixbuf_at_size(mainw->current_file, mainw->framedraw_frame, img_ext, tc,
                                        (double)cfile->hsize, (double)cfile->vsize,
                                        LIVES_INTERP_BEST);
+#ifdef TEST_GAMMA
+    // if pixbuf was from png it might already be corrected ?
+    gamma_correct_pixbuf(TRUE, prefs->screen_gamma, pixbuf);
+#endif
   }
 
   if (pixbuf != NULL) {
@@ -622,7 +628,6 @@ void load_framedraw_image(LiVESPixbuf *pixbuf) {
     mainw->fd_layer_orig = weed_layer_create(0, 0, NULL, WEED_PALETTE_END);
 
     if (!pixbuf_to_layer(mainw->fd_layer_orig, pixbuf)) lives_object_unref(pixbuf);
-
   }
 
   if (mainw->fd_layer != NULL) weed_layer_free(mainw->fd_layer);
@@ -692,7 +697,6 @@ boolean on_framedraw_enter(LiVESWidget *widget, LiVESXEventCrossing *event, live
 
   switch (framedraw->type) {
   case LIVES_PARAM_SPECIAL_TYPE_RECT_DEMASK:
-  case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTRECT:
   case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT:
     if (mainw->multitrack == NULL) {
       lives_set_cursor_style(LIVES_CURSOR_TOP_LEFT_CORNER, mainw->framedraw);
@@ -728,6 +732,8 @@ boolean on_framedraw_leave(LiVESWidget *widget, LiVESXEventCrossing *event, live
 boolean on_framedraw_mouse_start(LiVESWidget *widget, LiVESXEventButton *event, lives_special_framedraw_rect_t *framedraw) {
   // user clicked in the framedraw widget (or multitrack playback widget)
 
+  double xend, yend;
+
   int fd_height;
   int fd_width;
 
@@ -753,8 +759,7 @@ boolean on_framedraw_mouse_start(LiVESWidget *widget, LiVESXEventButton *event,
   lives_widget_get_pointer((LiVESXDevice *)mainw->mgeom[prefs->gui_monitor > 0 ? prefs->gui_monitor - 1 : 0].mouse_device,
                            widget, &xstarti, &ystarti);
 
-  if ((framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTRECT ||
-       framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT ||
+  if ((framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT ||
        framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_DEMASK) &&
       (mainw->multitrack == NULL || mainw->multitrack->cursor_style == 0)) {
     if (mainw->multitrack == NULL) {
@@ -775,14 +780,13 @@ boolean on_framedraw_mouse_start(LiVESWidget *widget, LiVESXEventButton *event,
   xstart /= (double)width;
   ystart /= (double)height;
 
-  xcurrent = xstart;
-  ycurrent = ystart;
+  xend = xcurrent = xstart;
+  yend = ycurrent = ystart;
 
   noupdate = TRUE;
 
   switch (framedraw->type) {
   case LIVES_PARAM_SPECIAL_TYPE_SINGLEPOINT:
-
     if (framedraw->xstart_param->dp > 0)
       lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->xstart_param->widgets[0]), xstart);
     else
@@ -794,10 +798,10 @@ boolean on_framedraw_mouse_start(LiVESWidget *widget, LiVESXEventButton *event,
 
     break;
 
-  case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTRECT:
   case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT:
-  case LIVES_PARAM_SPECIAL_TYPE_RECT_DEMASK:
+    xend = yend = 0.;
 
+  case LIVES_PARAM_SPECIAL_TYPE_RECT_DEMASK:
     if (framedraw->xstart_param->dp > 0)
       lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->xstart_param->widgets[0]), xstart);
     else
@@ -807,19 +811,14 @@ boolean on_framedraw_mouse_start(LiVESWidget *widget, LiVESXEventButton *event,
     else
       lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->ystart_param->widgets[0]), (int)(ystart * (double)cfile->vsize + .5));
 
-    if (framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTRECT) {
-      lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->xend_param->widgets[0]), 0.);
-      lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->yend_param->widgets[0]), 0.);
-    } else {
-      if (framedraw->xend_param->dp > 0)
-        lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->xend_param->widgets[0]), xstart);
-      else
-        lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->xend_param->widgets[0]), (int)(xstart * (double)cfile->hsize + .5));
-      if (framedraw->xend_param->dp > 0)
-        lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->yend_param->widgets[0]), ystart);
-      else
-        lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->yend_param->widgets[0]), (int)(ystart * (double)cfile->vsize + .5));
-    }
+    if (framedraw->xend_param->dp > 0)
+      lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->xend_param->widgets[0]), xend);
+    else
+      lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->xend_param->widgets[0]), (int)(xend * (double)cfile->hsize + .5));
+    if (framedraw->xend_param->dp > 0)
+      lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->yend_param->widgets[0]), yend);
+    else
+      lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->yend_param->widgets[0]), (int)(yend * (double)cfile->vsize + .5));
 
     break;
   default:
@@ -875,7 +874,8 @@ boolean on_framedraw_mouse_update(LiVESWidget *widget, LiVESXEventMotion *event,
   noupdate = TRUE;
 
   switch (framedraw->type) {
-  case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTRECT: {
+  case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT: {
+    // end parameters provide a scale relative to the output channel
     double xscale, yscale;
 
     xscale = xcurrent - xstart;
@@ -910,12 +910,11 @@ boolean on_framedraw_mouse_update(LiVESWidget *widget, LiVESXEventMotion *event,
         lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->ystart_param->widgets[0]), (int)(ycurrent * (double)cfile->vsize + .5));
       }
     }
-
   }
   break;
-  case LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT:
-  case LIVES_PARAM_SPECIAL_TYPE_RECT_DEMASK:
 
+  case LIVES_PARAM_SPECIAL_TYPE_RECT_DEMASK:
+    // end parameters provide the absolute point, in output channel ratio
     if (xcurrent > xstart) {
       if (framedraw->xend_param->dp > 0)
         lives_spin_button_set_value(LIVES_SPIN_BUTTON(framedraw->xend_param->widgets[0]), xcurrent);
@@ -950,7 +949,6 @@ boolean on_framedraw_mouse_update(LiVESWidget *widget, LiVESXEventMotion *event,
 
   default:
     break;
-
   }
 
   if (mainw->framedraw_reset != NULL) {
@@ -977,8 +975,7 @@ boolean on_framedraw_mouse_reset(LiVESWidget *widget, LiVESXEventButton *event,
   if (framedraw == NULL) return FALSE;
   if (mainw->multitrack != NULL && mainw->multitrack->track_index == -1) return FALSE;
 
-  if ((framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTRECT ||
-       framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT ||
+  if ((framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_MULTIRECT ||
        framedraw->type == LIVES_PARAM_SPECIAL_TYPE_RECT_DEMASK) &&
       (mainw->multitrack == NULL || mainw->multitrack->cursor_style == 0)) {
     if (mainw->multitrack == NULL) {
