@@ -28,10 +28,6 @@ struct _dvgrabw *create_camwindow(s_cam *cam, int type) {
   dvgrabw->dialog = lives_standard_dialog_new(_("DVGrab"), FALSE, -1, -1);
   dvgrabw->playing = FALSE;
 
-  if (prefs->show_gui) {
-    lives_window_set_transient_for(LIVES_WINDOW(dvgrabw->dialog), LIVES_WINDOW(LIVES_MAIN_WINDOW_WIDGET));
-  }
-
   vbox = lives_dialog_get_content_area(LIVES_DIALOG(dvgrabw->dialog));
 
   hbox = lives_hbox_new(FALSE, 0);
@@ -83,6 +79,8 @@ struct _dvgrabw *create_camwindow(s_cam *cam, int type) {
 
   lives_box_pack_start(LIVES_BOX(vbox), hbuttonbox1, FALSE, FALSE, widget_opts.packing_height);
 
+  // TODO: use lives_dialog_add_button_from_stock()
+
   button3 = lives_standard_button_new_from_stock(LIVES_STOCK_MEDIA_REWIND, NULL);
 
   lives_container_add(LIVES_CONTAINER(hbuttonbox1), button3);
