@@ -61,13 +61,27 @@ POSSIBILITY OF SUCH DAMAGES.
 
 #ifndef GUI_QT
 #define GUI_GTK
-#define PAINTER_CAIRO
+#define LIVES_PAINTER_IS_CAIRO
+#define LIVES_LINGO_IS_PANGO
 #else
 #define PAINTER_QPAINTER
 #define NO_PROG_LOAD
 #undef ENABLE_GIW
 #endif
 
+#include <sys/types.h>
+#include <sys/file.h>
+#include <unistd.h>
+
+typedef pid_t lives_pid_t;
+typedef int lives_pgid_t;
+
+#ifdef GUI_GTK
+#ifndef GDK_WINDOWING_X11
+#define GDK_WINDOWING_X11
+#endif
+#endif // GUI_GTK
+
 #ifdef GUI_GTK
 
 #define USE_GLIB
@@ -79,6 +93,8 @@ POSSIBILITY OF SUCH DAMAGES.
 #ifdef ENABLE_GIW
 #define ENABLE_GIW_3
 #endif
+#else
+#undef ENABLE_GIW_3
 #endif
 
 #endif
@@ -98,45 +114,6 @@ POSSIBILITY OF SUCH DAMAGES.
 #endif
 #endif
 
-#ifdef IS_MINGW
-
-#ifndef WINVER
-#define WINVER 0x0500
-#endif
-
-#include <windows.h>
-#include <winbase.h>
-#include <tlhelp32.h>
-#include <sddl.h>
-
-#define O_SYNC (FILE_FLAG_NO_BUFFERING|FILE_FLAG_WRITE_THROUGH)
-
-typedef PROCESS_INFORMATION *lives_pid_t;
-typedef PROCESS_INFORMATION *lives_pgid_t;
-
-#ifdef GUI_GTK
-#ifndef GDK_WINDOWING_WIN32
-#define GDK_WINDOWING_WIN32
-#endif
-#endif
-
-#else // IS_MINGW
-
-#ifdef GUI_GTK
-#ifndef GDK_WINDOWING_X11
-#define GDK_WINDOWING_X11
-#endif
-#else
-#include <sys/types.h>
-#include <unistd.h>
-
-#endif // GUI_GTK
-
-typedef pid_t lives_pid_t;
-typedef int lives_pgid_t;
-
-#endif // IS_MINGW
-
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -168,12 +145,7 @@ typedef int lives_pgid_t;
 /// this must match AC_PREFIX_DEFAULT in configure.in
 /// TODO - when lives-plugins is a separate package, use pkg-config to get PREFIX and remove PREFIX_DEFAULT
 #ifndef PREFIX_DEFAULT
-#ifndef IS_MINGW
 #define PREFIX_DEFAULT "/usr"
-#else
-// TODO - get this from the installer
-#define PREFIX_DEFAULT "C:\\Program Files\\LiVES"
-#endif
 #endif
 
 /// if --prefix= was not set, this is set to "NONE"
@@ -181,7 +153,6 @@ typedef int lives_pgid_t;
 #define PREFIX PREFIX_DEFAULT
 #endif
 
-#ifndef IS_MINGW
 #define LIVES_DIR_SEP "/"
 #define LIVES_STATUS_FILE_NAME ".status"
 #define LIVES_INFO_FILE_NAME ".info"
@@ -200,39 +171,16 @@ typedef int lives_pgid_t;
 #define PLUGIN_COMPOUND_DIR "/share/lives/plugins/"
 #define PLUGIN_EXEC_DIR "/lives/plugins/"
 #define ICON_DIR "/share/lives/icons/"
-#define DESKTOP_ICON_DIR "/share/icons/hicolor/48x48/apps"
+#define DESKTOP_ICON_DIR "/share/icons/hicolor/256x256/apps"
 #define DATA_DIR "/share/lives/"
+#define LIVES_RC_FILENAME ".lives"
 #define LIVES_CONFIG_DIR ".lives-dir/"
 #define LIVES_DEVICEMAP_DIR "devicemaps"
 #define LIVES_WORK_NAME "livesprojects"
 
-#else // IS_MINGW
-#define LIVES_DIR_SEP "\\"
-#define LIVES_STATUS_FILE_NAME "status"
-#define LIVES_INFO_FILE_NAME "info"
-#define LIVES_BFILE_NAME "smogrify"
-#define LIVES_SMOGPLUGIN_FILE_NAME "smogplugin"
-#define LIVES_SMOGVAL_FILE_NAME "smogval"
-#define LIVES_ENC_DEBUG_FILE_NAME "debug_out"
-#define LIVES_DEVNULL "NUL"
-
-#define DLL_NAME "dll"
-
-#define DOC_DIR "\\Documents/"
-
-#define THEME_DIR "\\Themes/"
-#define PLUGIN_SCRIPTS_DIR "\\Plugins/"
-#define PLUGIN_COMPOUND_DIR "\\Plugins/"
-#define PLUGIN_EXEC_DIR "\\Plugins/"
-#define ICON_DIR "\\Icons/"
-#define DATA_DIR "\\Data/"
-#define LIVES_CONFIG_DIR "\\Config/"
-#define LIVES_WORK_NAME "livesprojects"
-#endif
-
 #define LIVES_DEVICE_DIR "/dev/"
 
-#define LIVES_COPYRIGHT_YEARS "2002 - 2018"
+#define LIVES_COPYRIGHT_YEARS "2002 - 2019"
 
 #define LIVES_WEBSITE PACKAGE_URL
 #define LIVES_MANUAL_URL LIVES_WEBSITE "/manual/"
@@ -254,6 +202,8 @@ typedef int lives_pgid_t;
 
 #define DEF_FILE_PERMS (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH) ///< non-executable, is modified by the umask
 
+#define ALLOW_NONFREE_CODECS
+
 /// LiVES will show a warning if this (MBytes) is exceeded on load
 /// (can be overridden in prefs)
 #define WARN_FILE_SIZE 500
@@ -303,15 +253,19 @@ typedef int lives_pgid_t;
 #ifdef __GNUC__
 #  define WARN_UNUSED  __attribute__((warn_unused_result))
 #  define GNU_PURE  __attribute__((pure))
+#  define GNU_DEPRECATED(msg)  __attribute__((deprecated(msg)))
 #  define GNU_CONST  __attribute__((const))
 #  define GNU_MALLOC  __attribute__((malloc))
 #  define GNU_ALIGN(x) __attribute__((alloc_align(x)))
+#  define GNU_NORETURN __attribute__((noreturn))
 #else
 #  define WARN_UNUSED
 #  define GNU_PURE
 #  define GNU_CONST
 #  define GNU_MALLOC
+#  define GNU_DEPRECATED(msg)
 #  define GNU_ALIGN(x)
+#  define GNU_NORETURN
 #endif
 
 #ifdef PRODUCE_LOG
@@ -319,7 +273,36 @@ typedef int lives_pgid_t;
 #define LIVES_LOG "lives.log"
 #endif
 
+// math macros / functions
+
+// round to nearest integer
+#define ROUND_I(a) ((int)((double)a + .5))
+
+// clamp a between 0 and b; both values rounded to nearest int
+#define NORMAL_CLAMP(a, b) (ROUND_I(a)  < 0 ? 0 : ROUND_I(a) > ROUND_I(b) ? ROUND_I(b) : ROUND_I(a))
+
+// clamp a between 1 and b; both values rounded to nearest int; if rounded value of a is <= 0, return rounded b
+#define UTIL_CLAMP(a, b) (NORMAL_CLAMP(a, b) <= 0 ? ROUND_I(b) : ROUND_I(a))
+
+// round a up to next (integer) multiple of b
+#define CEIL(a, b) ((int)(((double)a + (double)b - .000000001) / ((double)b)) * b)
+
+// floating point division, maintains the sign of the dividend
+#define SIGNED_DIVIDE(a, b) (a < 0. ? (a / b > 0. ? -a / b : a / b) : (a / b < 0. ? -a / b : a / b))
+
+#ifdef NEED_ENDIAN_TEST
+#undef NEED_ENDIAN_TEST
+static int32_t testint = 0x12345678;
+#define IS_BIG_ENDIAN (((char *)&testint)[0] == 0x12)
+#endif
+
+// utils.c math functions
+void lives_srandom(unsigned int seed);
 uint64_t lives_random(void);
+float LEFloat_to_BEFloat(float f) GNU_CONST;
+uint64_t lives_10pow(int pow) GNU_CONST;
+double lives_fix(double val, int decimals) GNU_CONST;
+int get_approx_ln(uint32_t val) GNU_CONST;
 
 typedef struct {
   uint16_t red;
@@ -388,6 +371,8 @@ typedef struct {
 #include "lbindings.h"
 #endif
 
+#define N_RECENT_FILES 4
+
 typedef enum {
   UNDO_NONE = 0,
   UNDO_EFFECT,
@@ -458,6 +443,9 @@ typedef enum {
   /// generator was stopped
   CANCEL_GENERATOR_END,
 
+  /// external process ended (e.g. autolives uncheck)
+  CANCEL_EXTERNAL_ENDED,
+
   /// user pressed 'Keep'
   CANCEL_KEEP,
 
@@ -470,6 +458,9 @@ typedef enum {
   /// cancelled and paused
   CANCEL_USER_PAUSED,
 
+  /// an error occured, retry the operation
+  CANCEL_RETRY,
+
   /// special cancel for TV toy
   CANCEL_KEEP_LOOPING = CANCEL_NONE + 100
 } lives_cancel_t;
@@ -521,10 +512,10 @@ typedef enum {
 #define IS_VALID_CLIP(clip) (clip >= 0 && mainw->files[clip] != NULL)
 #define CURRENT_CLIP_IS_VALID IS_VALID_CLIP(mainw->current_file)
 
-#define CLIP_HAS_VIDEO(clip) (IS_VALID_CLIP(clip) && mainw->files[clip]->frames > 0)
+#define CLIP_HAS_VIDEO(clip) (IS_VALID_CLIP(clip) ? mainw->files[clip]->frames > 0 : FALSE)
 #define CURRENT_CLIP_HAS_VIDEO CLIP_HAS_VIDEO(mainw->current_file)
 
-#define CLIP_HAS_AUDIO(clip) (IS_VALID_CLIP(clip) && mainw->files[clip]->achans > 0 && mainw->files[clip]->asampsize > 0)
+#define CLIP_HAS_AUDIO(clip) (IS_VALID_CLIP(clip) ? mainw->files[clip]->achans > 0 && mainw->files[clip]->asampsize > 0 : FALSE)
 #define CURRENT_CLIP_HAS_AUDIO CLIP_HAS_AUDIO(mainw->current_file)
 
 #define CLIP_VIDEO_TIME(clip) ((double)(IS_VALID_CLIP(clip) ? mainw->files[clip]->video_time : 0.))
@@ -537,28 +528,35 @@ typedef enum {
 
 #define CLIP_TOTAL_TIME(clip) ((double)(CLIP_VIDEO_TIME(clip) > CLIP_AUDIO_TIME(clip) ? CLIP_VIDEO_TIME(clip) : CLIP_AUDIO_TIME(clip)))
 
+#define LIVES_IS_PLAYING (mainw->playing_file > -1)
+
 #define CURRENT_CLIP_TOTAL_TIME CLIP_TOTAL_TIME(mainw->current_file)
 
+#define CURRENT_CLIP_IS_CLIPBOARD (mainw->current_file == 0)
+
 /// corresponds to one clip in the GUI
 typedef struct {
   // basic info (saved during backup)
-  int bpp;
-  double fps;
-  int hsize; ///< in pixels (NOT macropixels !)
-  int vsize;
-  int arps; ///< audio sample rate
+  int bpp; ///< bits per pixel of the image frames, 24 or 32
+  double fps;  /// framerate of the clip
+  int hsize; ///< frame width (horizontal) in pixels (NOT macropixels !)
+  int vsize; ///< frame height (vertical) in pixels
+  int arps; ///< audio sample rate (the actual physical sample rate of the medium)
   uint32_t signed_endian; ///< bitfield
 
-  int arate; ///< audio playback rate
+  int arate; ///< current audio playback rate (may vary from arps), can even be 0 or negative
   uint64_t unique_id;    ///< this and the handle can be used to uniquely id a file
-  int achans;
-  int asampsize;
+  int achans; ///< number of audio channels (0, 1 or 2)
+  int asampsize; ///< audio sample size in bits (8 or 16)
 
   /////////////////
-  int frames;
+  int frames;  ///< number of video frames
+
+  // TODO: make 1024 (to handle flv for example)
   char title[256];
   char author[256];
   char comment[256];
+
   char keywords[256];
   ////////////////
 
@@ -569,8 +567,6 @@ typedef struct {
 
 #define LIVES_CLIP_HEADER_VERSION 100
 
-  int rowstride;
-
   /// the processing window
   xprocess *proc_ptr;
 
@@ -597,9 +593,10 @@ typedef struct {
   boolean orig_file_name;
   boolean was_renamed;
   boolean is_untitled;
-  double pb_fps;
-  double freeze_fps;
+  double pb_fps;  ///< current playback rate, may vary from fps, can be 0. or negative
+  double freeze_fps; ///< pb_fps for paused / frozen clips
   boolean play_paused;
+  boolean was_in_set;
 
   //opening/restoring status
   boolean opening;
@@ -619,7 +616,7 @@ typedef struct {
   double video_time; // TODO: deprecate, calculate CLIP_VIDEO_TIME from frames and fps
   double laudio_time;
   double raudio_time;
-  double pointer_time;
+  double pointer_time;  ///< pointer time in timeline, also the playback start position for clipeditor (unless playing the selection)
 
   // used only for insert_silence, holds pre-padding length for undo
   double old_laudio_time;
@@ -633,7 +630,7 @@ typedef struct {
   // see resample.c for new events system
 
   // events
-  event *events[1];  ///<for block resampler
+  event *resample_events;  ///<for block resampler
 
   weed_plant_t *event_list;
   weed_plant_t *event_list_back;
@@ -725,6 +722,7 @@ typedef struct {
   boolean needs_update; ///< loaded values were incorrect, update header
 
   float **audio_waveform; ///< values for drawing the audio wave
+  int *aw_sizes; ///< size of each audio_waveform in floats
 } lives_clip_t;
 
 typedef struct {
@@ -733,6 +731,7 @@ typedef struct {
   boolean smog_version_correct;
   boolean can_read_from_config;
   boolean can_write_to_config;
+  boolean can_write_to_config_new;
   boolean can_write_to_home;
   boolean can_write_to_workdir;
 
@@ -756,6 +755,8 @@ typedef struct {
   boolean has_gdb;
   boolean has_gconftool_2;
   boolean has_xdg_screensaver;
+  boolean has_ssh_askpass;
+  boolean has_youtube_dl;
 
   /// home directory - default location for config file - locale encoding
   char home_dir[PATH_MAX];
@@ -787,8 +788,7 @@ typedef struct {
   char *myname_full;
   char *myname;
 
-  boolean has_stderr;
-
+  int xstdout;
   int nmonitors;
 
   int ncpus;
@@ -803,7 +803,7 @@ typedef struct {
 /// some shared structures
 extern capability *capable;
 
-#ifdef HAVE_JACK_JACK_H
+#ifdef ENABLE_JACK
 #include "jack.h"
 #endif
 
@@ -884,19 +884,25 @@ typedef enum {
 // dialogs.c
 boolean do_progress_dialog(boolean visible, boolean cancellable, const char *text);
 boolean do_warning_dialog(const char *text);
+boolean do_warning_dialogf(const char *fmt, ...);
 boolean do_warning_dialog_with_check(const char *text, int warn_mask_number);
 boolean do_warning_dialog_with_check_transient(const char *text, int warn_mask_number, LiVESWindow *transient);
 boolean do_yesno_dialog(const char *text);
+boolean do_yesno_dialogf(const char *fmt, ...);
 boolean do_yesno_dialog_with_check(const char *text, int warn_mask_number);
 boolean do_yesno_dialog_with_check_transient(const char *text, int warn_mask_number, LiVESWindow *transient);
 boolean do_yesno_dialog_with_check(const char *text, int warn_mask_number);
 boolean do_yesno_dialog_with_check_transient(const char *text, int warn_mask_number, LiVESWindow *transient);
 int do_abort_cancel_retry_dialog(const char *text, LiVESWindow *transient) WARN_UNUSED;
 int do_error_dialog(const char *text);
+int do_error_dialogf(const char *fmt, ...);
 int do_info_dialog(const char *text);
+int do_info_dialogf(const char *fmt, ...);
 int do_error_dialog_with_check(const char *text, int warn_mask_number);
 int do_blocking_error_dialog(const char *text);
+int do_blocking_error_dialogf(const char *fmt, ...);
 int do_blocking_info_dialog(const char *text);
+int do_blocking_info_dialogf(const char *fmt, ...);
 int do_error_dialog_with_check_transient(const char *text, boolean is_blocking, int warn_mask_number,
     LiVESWindow *transient);
 int do_info_dialog_with_transient(const char *text, boolean is_blocking, LiVESWindow *transient);
@@ -954,6 +960,7 @@ void do_aud_during_play_error(void);
 void do_rendered_fx_dialog(void);
 void do_layout_scrap_file_error(void);
 void do_layout_ascrap_file_error(void);
+void do_program_not_found_error(const char *progname);
 void do_lb_composite_error(void);
 void do_lb_convert_error(void);
 void do_ra_convert_error(void);
@@ -962,6 +969,7 @@ boolean do_set_duplicate_warning(const char *new_set);
 boolean do_set_rename_old_layouts_warning(const char *new_set);
 boolean do_layout_alter_frames_warning(void);
 boolean do_layout_alter_audio_warning(void);
+boolean do_reload_set_query(void);
 boolean do_yuv4m_open_warning(void);
 void do_mt_undo_mem_error(void);
 void do_mt_undo_buf_error(void);
@@ -977,8 +985,9 @@ void do_vpp_palette_error(void);
 void do_vpp_fps_error(void);
 void do_decoder_palette_error(void);
 void do_rmem_max_error(int size);
-boolean do_original_lost_warning(const char *fname);
+int do_original_lost_warning(const char *fname);
 void do_no_decoder_error(const char *fname);
+void do_no_loadfile_error(const char *fname);
 void do_jack_noopen_warn(void);
 void do_jack_noopen_warn2(void);
 void do_jack_noopen_warn3(void);
@@ -986,7 +995,7 @@ void do_jack_noopen_warn4(void);
 void do_file_perm_error(const char *file_name);
 void do_dir_perm_error(const char *dir_name);
 void do_dir_perm_access_error(const char *dir_name);
-void do_encoder_img_ftm_error(render_details *rdet);
+void do_encoder_img_fmt_error(render_details *rdet);
 void do_after_crash_warning(void);
 void do_bad_layout_error(void);
 void do_card_in_use_error(void);
@@ -996,7 +1005,7 @@ void do_invalid_subs_error(void);
 boolean do_erase_subs_warning(void);
 boolean do_sub_type_warning(const char *ext, const char *type_ext);
 boolean do_move_workdir_dialog(void);
-void do_set_locked_warning(const char *setname);
+boolean do_set_locked_warning(const char *setname);
 void do_no_in_vdevs_error(void);
 void do_locked_in_vdevs_error(void);
 void do_do_not_close_d(void);
@@ -1010,8 +1019,8 @@ void do_bad_theme_error(const char *themefile);
 void do_bad_theme_import_error(const char *theme_file);
 boolean do_theme_exists_warn(const char *themename);
 
-boolean process_one(boolean visible);
-void do_threaded_dialog(char *translated_text, boolean has_cancel);
+int process_one(boolean visible);
+void do_threaded_dialog(const char *translated_text, boolean has_cancel);
 void end_threaded_dialog(void);
 void threaded_dialog_spin(double fraction);
 void response_ok(LiVESButton *button, livespointer user_data);
@@ -1019,6 +1028,13 @@ void pump_io_chan(LiVESIOChannel *iochan);
 
 void do_splash_progress(void);
 
+// message collection
+void d_print(const char *fmt, ...);
+char *dump_messages(int start, int end); // utils.c
+weed_plant_t *get_nth_info_message(int n); // utils.c
+int add_messages_to_list(const char *text);
+int free_n_msgs(int frval);
+
 // d_print shortcuts
 void d_print_cancelled(void);
 void d_print_failed(void);
@@ -1042,6 +1058,7 @@ void open_fw_device(void);
 char *get_untitled_name(int number);
 boolean get_new_handle(int index, const char *name);
 boolean get_temp_handle(int index, boolean create);
+int close_temp_handle(int clipno, int new_clip);
 boolean get_handle_from_info_file(int index);
 void create_cfile(void);
 void save_file(int clip, int start, int end, const char *filename);
@@ -1051,7 +1068,7 @@ boolean save_frame_inner(int clip, int frame, const char *file_name, int width,
 void wait_for_stop(const char *stop_command);
 boolean save_clip_values(int which_file);
 void add_to_recovery_file(const char *handle);
-void rewrite_recovery_file(void);
+boolean rewrite_recovery_file(void);
 boolean check_for_recovery_files(boolean auto_recover);
 void recover_layout_map(int numclips);
 const char *get_deinterlace_string(void);
@@ -1066,15 +1083,15 @@ ulong restore_file(const char *filename);
 boolean read_headers(const char *file_name);
 
 // saveplay.c sets
-void open_set_file(const char *set_name, int clipnum);
+void open_set_file(int clipnum);
 
 // saveplay.c scrap file
 boolean open_scrap_file(void);
 boolean open_ascrap_file(void);
 int save_to_scrap_file(weed_plant_t *layer);
 boolean load_from_scrap_file(weed_plant_t *layer, int frame);
-void close_ascrap_file(void);
-void close_scrap_file(void);
+void close_ascrap_file(boolean remove);
+void close_scrap_file(boolean remove);
 void add_to_ascrap_mb(uint64_t bytes);
 
 boolean check_for_disk_space(void);
@@ -1085,7 +1102,7 @@ typedef void (*SignalHandlerPointer)(int);
 void set_signal_handlers(SignalHandlerPointer sigfunc);
 void catch_sigint(int signum);
 void defer_sigint(int signum);
-boolean startup_message_fatal(const char *msg);
+void startup_message_fatal(const char *msg) GNU_NORETURN;
 boolean startup_message_choice(const char *msg, int msgtype);
 boolean startup_message_nonfatal(const char *msg);
 boolean startup_message_info(const char *msg);
@@ -1140,21 +1157,22 @@ void set_colours(LiVESWidgetColor *colf, LiVESWidgetColor *colb, LiVESWidgetColo
 void set_preview_box_colours(void);
 void load_theme_images(void);
 void set_interactive(boolean interactive);
-char *get_menu_name(lives_clip_t *sfile);
+char *get_menu_name(lives_clip_t *sfile, boolean add_set);
 void enable_record(void);
 void toggle_record(void);
 void disable_record(void);
 void make_custom_submenus(void);
 void fade_background(void);
 void unfade_background(void);
+void fullscreen_internal(void);
 void block_expose(void);
 void unblock_expose(void);
 void frame_size_update(void);
 void splash_init(void);
 void splash_end(void);
 void splash_msg(const char *msg, double pct);
-void add_message_scroller(LiVESWidget *conter);
 void resize_widgets_for_monitor(boolean get_play_times);
+void reset_message_area(boolean expand);
 #if GTK_CHECK_VERSION(3, 0, 0)
 void calibrate_sepwin_size(void);
 boolean expose_pim(LiVESWidget *, lives_painter_t *, livespointer);
@@ -1164,6 +1182,7 @@ boolean expose_eim(LiVESWidget *, lives_painter_t *, livespointer);
 
 // system calls in utils.c
 int lives_system(const char *com, boolean allow_error);
+size_t lives_popen(const char *com, boolean allow_error, char *buff, size_t buflen);
 lives_pid_t lives_fork(const char *com);
 int lives_open3(const char *pathname, int flags, mode_t mode);
 int lives_open2(const char *pathname, int flags);
@@ -1177,7 +1196,6 @@ ssize_t lives_write(int fd, livesconstpointer buf, size_t count, boolean allow_f
 ssize_t lives_write_buffered(int fd, const char *buf, size_t count, boolean allow_fail);
 ssize_t lives_write_le(int fd, livesconstpointer buf, size_t count, boolean allow_fail);
 ssize_t lives_write_le_buffered(int fd, livesconstpointer buf, size_t count, boolean allow_fail);
-ssize_t file_buffer_flush(int fd);
 ssize_t lives_read(int fd, void *buf, size_t count, boolean allow_less);
 ssize_t lives_read_buffered(int fd, void *buf, size_t count, boolean allow_less);
 ssize_t lives_read_le(int fd, void *buf, size_t count, boolean allow_less);
@@ -1185,24 +1203,19 @@ ssize_t lives_read_le_buffered(int fd, void *buf, size_t count, boolean allow_le
 int lives_chdir(const char *path, boolean allow_fail);
 int lives_fputs(const char *s, FILE *stream);
 char *lives_fgets(char *s, int size, FILE *stream);
+size_t lives_fread(void *ptr, size_t size, size_t nmemb, FILE *stream);
 pid_t lives_getpid(void);
 int lives_getgid(void);
 int lives_getuid(void);
 boolean lives_freep(void **ptr);
 void lives_kill_subprocesses(const char *dirname, boolean kill_parent);
 void lives_suspend_resume_process(const char *dirname, boolean suspend);
-#ifdef IS_MINGW
-boolean lives_win32_suspend_resume_process(DWORD pid, boolean suspend);
-boolean lives_win32_kill_subprocesses(DWORD pid, boolean kill_parent);
-int lives_win32_get_num_logical_cpus(void);
-#endif
 int lives_kill(lives_pid_t pid, int sig);
 int lives_killpg(lives_pgid_t pgrp, int sig);
-void lives_srandom(unsigned int seed);
 ssize_t lives_readlink(const char *path, char *buf, size_t bufsiz);
 boolean lives_setenv(const char *name, const char *value);
 boolean lives_fsync(int fd);
-void lives_sync(void);
+void lives_sync(int times);
 int lives_rmdir(const char *dir, boolean force);
 int lives_rmdir_with_parents(const char *dir);
 int lives_rm(const char *file);
@@ -1218,15 +1231,16 @@ int lives_echo(const char *text, const char *to, boolean append);
 int lives_ln(const char *from, const char *to);
 
 int lives_utf8_strcasecmp(const char *s1, const char *s2);
+int lives_utf8_strcmp(const char *s1, const char *s2);
+
+boolean lives_string_ends_with(const char *string, const char *fmt, ...);
 
 char *filename_from_fd(char *val, int fd);
 
-float LEFloat_to_BEFloat(float f) GNU_CONST;
-uint64_t lives_10pow(int pow) GNU_CONST;
-double lives_fix(double val, int decimals) GNU_CONST;
-int get_approx_ln(uint32_t val) GNU_CONST;
+int64_t lives_get_relative_ticks(int64_t origsecs, int64_t origusecs);
+int64_t lives_get_current_playback_ticks(int64_t origsecs, int64_t origusecs, lives_time_source_t *time_source);
+int64_t lives_get_current_ticks(void);
 
-int64_t lives_get_current_ticks(int64_t delta_seconds, int64_t delta_usec);
 boolean lives_alarm_get(int alarm_handle);
 int lives_alarm_set(int64_t ticks);
 void lives_alarm_clear(int alarm_handle);
@@ -1234,7 +1248,7 @@ lives_storage_status_t get_storage_status(const char *dir, uint64_t warn_level,
 char *lives_format_storage_space_string(uint64_t space);
 char *lives_datetime(struct timeval *tv);
 
-int myround(double n);
+int myround(double n) GNU_CONST; // round up +ve or down -ve
 void get_dirname(char *filename);
 char *get_dir(const char *filename);
 void get_basename(char *filename);
@@ -1242,7 +1256,6 @@ void get_filename(char *filename, boolean strip_dir);
 char *get_extension(const char *filename);
 uint64_t get_version_hash(const char *exe, const char *sep, int piece);
 uint64_t make_version_hash(const char *ver);
-void d_print(const char *fmt, ...);
 void init_clipboard(void);
 boolean cache_file_contents(const char *filename);
 char *get_val_from_cached_list(const char *key, size_t maxlen);
@@ -1254,35 +1267,30 @@ const char *get_image_ext_for_type(lives_image_type_t imgtype);
 lives_image_type_t lives_image_ext_to_type(const char *img_ext);
 lives_image_type_t lives_image_type_to_image_type(const char *lives_img_type);
 
-void set_menu_text(LiVESWidget *menu, const char *text, boolean use_mnemonic);
-void get_menu_text(LiVESWidget *menu, char *text);
 void reset_clipmenu(void);
 
-double lives_ce_update_timeline(int frame, double x);  ///< pointer position in timeline
-void get_play_times(void); ///< recalculate video / audio lengths and draw the timer bars
-void update_play_times(void); ///< like get_play_times, but will force redraw audio waveforms
-void update_timer_bars(int posx, int posy, int width, int height, int which); ///< draw the timer bars
-void redraw_timer_bars(double oldx, double newx, int which); ///< paint a damage region
 void get_total_time(lives_clip_t *file); ///< calculate laudio, raudio and video time (may be deprecated and replaced with macros)
+void get_play_times(void); ///< recalculate video / audio lengths and draw the timer bars
+void update_play_times(void); ///< like get_play_times, but will force redraw of audio waveforms
+
+uint32_t get_signed_endian(boolean is_signed, boolean little_endian); ///< produce bitmapped value
+
+void switch_aud_to_none(boolean set_pref);
+boolean switch_aud_to_sox(boolean set_pref);
+boolean switch_aud_to_jack(boolean set_pref);
+boolean switch_aud_to_pulse(boolean set_pref);
 
-uint32_t get_signed_endian(boolean is_signed, boolean little_endian);
-void fullscreen_internal(void);
-void switch_to_int_player(void);
-void switch_to_mplayer(void);
-void switch_aud_to_sox(boolean set_pref);
-boolean switch_aud_to_jack(void);
-boolean switch_aud_to_pulse(void);
-void switch_aud_to_mplayer(boolean set_pref);
-void switch_aud_to_mplayer2(boolean set_pref);
 boolean prepare_to_play_foreign(void);
 boolean after_foreign_play(void);
 boolean check_file(const char *file_name, boolean check_exists);  ///< check if file exists
 boolean check_dir_access(const char *dir);
+boolean lives_make_writeable_dir(const char *newdir);
 uint64_t get_file_size(int fd);
 uint64_t sget_file_size(const char *name);
 uint64_t get_fs_free(const char *dir);
 boolean is_writeable_dir(const char *dir);
 boolean ensure_isdir(char *fname);
+boolean dirs_equal(const char *dira, const char *dirb);
 char *ensure_extension(const char *fname, const char *ext) WARN_UNUSED;
 boolean check_dev_busy(char *devstr);
 void activate_url_inner(const char *link);
@@ -1290,10 +1298,11 @@ void activate_url(LiVESAboutDialog *about, const char *link, livespointer data);
 void show_manual_section(const char *lang, const char *section);
 
 double calc_time_from_frame(int clip, int frame);
-int calc_frame_from_time(int filenum, double time);   ///< nearest frame start
-int calc_frame_from_time2(int filenum, double time);  ///< nearest frame end
-int calc_frame_from_time3(int filenum, double time);  ///< nearest frame mid
-int calc_frame_from_time3(int filenum, double time);  ///< nearest frame start, no maximum
+int calc_frame_from_time(int filenum, double time);   ///< nearest frame [1, frames]
+int calc_frame_from_time2(int filenum, double time);  ///< nearest frame [1, frames+1]
+int calc_frame_from_time3(int filenum, double time);  ///< nearest frame rounded down, [1, frames+1]
+int calc_frame_from_time4(int filenum, double time);  ///<  nearest frame, no maximum
+
 
 boolean check_for_ratio_fps(double fps);
 double get_ratio_fps(const char *string);
@@ -1329,7 +1338,6 @@ int verhash(char *version);
 void set_undoable(const char *what, boolean sensitive);
 void set_redoable(const char *what, boolean sensitive);
 void zero_spinbuttons(void);
-void draw_little_bars(double ptrtime, int which);
 void set_sel_label(LiVESWidget *label);
 void clear_mainw_msg(void);
 int get_token_count(const char *string, int delim);
@@ -1347,20 +1355,18 @@ LiVESList *lives_list_delete_string(LiVESList *, char *string) WARN_UNUSED;
 LiVESList *lives_list_copy_strings(LiVESList *list);
 boolean string_lists_differ(LiVESList *, LiVESList *);
 
-boolean is_realtime_aplayer(int ptype);
-
 LiVESList *get_set_list(const char *dir, boolean utf8);
 
 char *subst(const char *string, const char *from, const char *to);
 char *insert_newlines(const char *text, int maxwidth);
 
 int hextodec(char *string);
-int get_hex_digit(const char *c);
+int get_hex_digit(const char *c) GNU_CONST;
 
 uint32_t fastrand(void);
 void fastsrand(uint32_t seed);
 
-int lives_list_strcmp_index(LiVESList *list, livesconstpointer data);
+int lives_list_strcmp_index(LiVESList *list, livesconstpointer data, boolean case_sensitive);
 
 lives_cancel_t check_for_bad_ffmpeg(void);
 
@@ -1389,25 +1395,19 @@ boolean save_srt_subtitles(lives_clip_t *sfile, double start_time, double end_ti
 
 // inlines
 #define cfile mainw->files[mainw->current_file]
-#define clipboard mainw->files[0]
+#define CLIPBOARD_FILE 0
+#define clipboard mainw->files[CLIPBOARD_FILE]
 
 #define PREFS_TIMEOUT 10000000 ///< 10 seconds
 
 #define LIVES_TV_CHANNEL1 "http://www.serverwillprovide.com/sorteal/livestvclips/livestv.ogm"
 
-// round (double) a up to next (integer) multiple of (double) b
-#define CEIL(a, b) ((int)(((double)a + (double)b - .000000001) / ((double)b)) * b)
-
-#ifdef NEED_ENDIAN_TEST
-#undef NEED_ENDIAN_TEST
-static int32_t testint = 0x12345678;
-#define IS_BIG_ENDIAN (((char *)&testint)[0] == 0x12)
-#endif
-
 char *dummychar;
 
 void break_me(void);
 
+//#define TEST_GAMMA
+
 #define LIVES_NO_DEBUG
 #ifndef LIVES_DEBUG
 #ifndef LIVES_NO_DEBUG
