@@ -1,6 +1,6 @@
 // paramspecial.h
 // LiVES
-// (c) G. Finch 2004 - 2013 <salsaman@gmail.com>
+// (c) G. Finch 2004 - 2013 <salsaman+lives@gmail.com>
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -21,20 +21,21 @@ typedef struct {
   lives_param_t *xend_param;
   lives_param_t *yend_param;
 
-  int stdwidgets; // 2 for singlepoint, 4 for demask, multrect
+  int stdwidgets; // 2 for singlepoint, 4 for demask, multirect
   int *extra_params;
   int num_extra;
   LiVESWidget **extra_widgets;
 } lives_special_framedraw_rect_t;
 
-
 typedef struct {
   lives_param_t *height_param;
   lives_param_t *width_param;
-  LiVESWidget *checkbutton;
+  LiVESWidget *lockbutton;
   LiVESWidget *label;
   ulong width_func;
   ulong height_func;
+  double ratio;
+  int nwidgets;
 } lives_special_aspect_t;
 
 typedef struct {
@@ -44,6 +45,9 @@ typedef struct {
 } lives_special_mergealign_t;
 ////////////////////////////////
 
+#define ASPECT_BUTTON_WIDTH 32
+#define ASPECT_BUTTON_HEIGHT 32
+
 void init_special(void);
 
 void add_to_special(const char *special_string, lives_rfx_t *);
@@ -71,4 +75,3 @@ boolean is_perchannel_multi(lives_rfx_t *rfx, int pnum);
 lives_special_mergealign_t mergealign;
 
 #endif
-
