@@ -37,21 +37,20 @@ typedef struct {
   short audio_player;
 #define AUD_PLAYER_NONE 0
 #define AUD_PLAYER_SOX 1
-#define AUD_PLAYER_MPLAYER 2
-#define AUD_PLAYER_JACK 3
-#define AUD_PLAYER_PULSE 4
-#define AUD_PLAYER_MPLAYER2 5
+#define AUD_PLAYER_JACK 2
+#define AUD_PLAYER_PULSE 3
 
   // string forms
+#define AUDIO_PLAYER_NONE "none"
 #define AUDIO_PLAYER_SOX "sox"
 #define AUDIO_PLAYER_JACK "jack"
 #define AUDIO_PLAYER_PULSE "pulse"
-#define AUDIO_PLAYER_PULSE_AUDIO "pulse audio" // display version
+#define AUDIO_PLAYER_PULSE_AUDIO "pulseaudio" // display version
 
 #define AUDIO_PLAYER_MPLAYER "mplayer"
 #define AUDIO_PLAYER_MPLAYER2 "mplayer2"
 
-  char aplayer[512]; // name, eg. "jack","pulse","sox","mplayer","mplayer2"
+  char aplayer[512]; // name, eg. "jack","pulse","sox"
 
   /// frame quantisation type
   short q_type;
@@ -126,7 +125,6 @@ typedef struct {
   char audio_play_command[PATH_MAX * 2];
   char cdplay_device[PATH_MAX];  ///< locale encoding
   double default_fps;
-  int bar_height;
   boolean pause_effect_during_preview;
   boolean open_decorated;
   int sleep_time;
@@ -175,10 +173,12 @@ typedef struct {
 
   boolean antialias;
 
-  boolean ignore_tiny_fps_diffs;
+  double fps_tolerance;
 
   short rte_keys_virtual;
 
+  boolean show_msg_area;
+
   uint32_t jack_opts;
 #define JACK_OPTS_TRANSPORT_CLIENT (1<<0)   ///< jack can start/stop
 #define JACK_OPTS_TRANSPORT_MASTER (1<<1)  ///< transport master
@@ -324,12 +324,15 @@ typedef struct {
   /** default 0; 1==use old (bad) behaviour on bigendian machines (r/w bigend ints/doubles); 2==bad reads, good writes */
   int bigendbug;
 
-  // these are defualt values; actual values can be adjusted in Preferences
-#define DEF_DS_WARN_LEVEL 250000000  // 250MB
+  // these are default values; actual values can be adjusted in Preferences
+#define DEF_DS_WARN_LEVEL 2500000000  // 2.5 GB
   uint64_t ds_warn_level; ///< diskspace warn level bytes
-#define DEF_DS_CRIT_LEVEL 20000000 // 20MB
+#define DEF_DS_CRIT_LEVEL 250000000 // 250MB
   uint64_t ds_crit_level; ///< diskspace critical level bytes
 
+#define DEF_MSG_TEXTSIZE 4 // LIVES_FONTSIZE_LARGE (via lives_textsize_to_string())
+#define DEF_MAX_MSGS 10000
+
 #define LIVES_CDISK_LEAVE_ORPHAN_SETS (1<<0)
 #define LIVES_CDISK_LEAVE_BFILES (1<<1)
 #define LIVES_CDISK_REMOVE_ORPHAN_LAYOUTS (1<<2)
@@ -377,6 +380,16 @@ typedef struct {
   boolean show_asrc;
 
   float ahold_threshold;
+
+  double screen_gamma;
+
+  int max_messages;
+
+  int msg_textsize;
+
+  double screen_scale;
+
+  char *def_workdir;
 } _prefs;
 
 enum {
@@ -412,6 +425,9 @@ enum {
 typedef struct {
   ulong encoder_ofmt_fn;
   ulong encoder_name_fn;
+  ulong close_func;
+  LiVESAccelGroup *accel_group;
+
   LiVESWidget *prefs_dialog;
 
   LiVESWidget *prefs_list;
@@ -445,7 +461,7 @@ typedef struct {
   LiVESWidget *scrollw_right_jack;
   LiVESWidget *scrollw_right_midi;
   LiVESWidget *right_shown;
-  LiVESWidget *cancelbutton;
+  LiVESWidget *revertbutton;
   LiVESWidget *applybutton;
   LiVESWidget *closebutton;
   LiVESWidget *stop_screensaver_check;
@@ -467,6 +483,7 @@ typedef struct {
   LiVESWidget *pbq_combo;
   LiVESWidget *ofmt_combo;
   LiVESWidget *audp_combo;
+  LiVESWidget *pa_gens;
   LiVESWidget *rframes;
   LiVESWidget *rfps;
   LiVESWidget *rclips;
@@ -515,6 +532,8 @@ typedef struct {
   LiVESWidget *cbutton_mab;
   LiVESWidget *cbutton_infot;
   LiVESWidget *cbutton_infob;
+  LiVESWidget *fb_filebutton;
+  LiVESWidget *se_filebutton;
   LiVESWidget *theme_style2;
   LiVESWidget *theme_style3;
   LiVESWidget *theme_style4;
@@ -536,6 +555,9 @@ typedef struct {
   LiVESWidget *jpeg;
   LiVESWidget *mt_enter_prompt;
   LiVESWidget *spinbutton_ocp;
+  LiVESWidget *nmessages_spin;
+  LiVESWidget *msgs_unlimited;
+  LiVESWidget *msg_textsize_combo;
   LiVESWidget *acodec_combo;
   LiVESWidget *spinbutton_osc_udp;
   LiVESWidget *spinbutton_rte_keys;
@@ -610,6 +632,7 @@ typedef struct {
   LiVESWidget *midi_hbox;
   LiVESWidget *frameblank_entry;
   LiVESWidget *sepimg_entry;
+  LiVESWidget *dialog_hpaned;
   LiVESTreeSelection *selection;
 
   boolean ignore_apply; ///< dont light the apply button when thing changes (for external calls), normally FALSE
@@ -627,6 +650,9 @@ typedef struct {
 } _ign_opts;
 
 typedef struct {
+  // if a pref also has an entry in future_prefs, be wary of changing its value
+  // seek to understand why it has a variant value
+
   char workdir[PATH_MAX];
   char theme[64];
   char vpp_name[64]; ///< new video playback plugin
@@ -638,6 +664,7 @@ typedef struct {
   int vpp_fwidth;
   int vpp_fheight;
   int vpp_argc;
+
   char **vpp_argv;
 
   _encoder encoder;
@@ -647,10 +674,14 @@ typedef struct {
   int startup_interface;
   uint32_t jack_opts;
 
+  int audio_src;
+
   int nfx_threads;
 
   LiVESList *disabled_decoders;
   LiVESList *disabled_decoders_new;
+
+  short sepwin_type;
 } _future_prefs;
 
 _prefs *prefs;
@@ -660,7 +691,7 @@ _prefsw *prefsw;
 void set_acodec_list_from_allowed(_prefsw *, render_details *);
 void  rdet_acodec_changed(LiVESCombo *acodec_combo, livespointer user_data);
 
-_prefsw *create_prefs_dialog(void);
+_prefsw *create_prefs_dialog(LiVESWidget *saved_dialog);
 
 boolean on_prefs_delete_event(LiVESWidget *, LiVESXEvent *, livespointer prefsw);
 
@@ -680,8 +711,8 @@ void populate_combo_box(LiVESCombo *, LiVESList *data);
 
 void set_combo_box_active_string(LiVESCombo *, char *active_str);
 
-void prefsw_set_astream_settings(_vid_playback_plugin *);
-void prefsw_set_rec_after_settings(_vid_playback_plugin *);
+void prefsw_set_astream_settings(_vid_playback_plugin *, _prefsw *);
+void prefsw_set_rec_after_settings(_vid_playback_plugin *, _prefsw *);
 
 void pref_change_images(void);
 void pref_change_xcolours(void);
@@ -692,9 +723,25 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 // NOTE: the following definitions must match with equivalent keys in smogrify
 // factories (pseudo prefs), called from cpp binding
 
+// TODO:
+/*typedef struct {
+  const char *pref_name;
+  int type;
+} lives_preference;
+
+const lives_preference [] = {
+  {PREF_REC_EXT_AUDIO, WEED_SEED_BOOL},
+};
+
+then:
+
+widget = lives_standard_widget_for_pref(const char *prefname, const char *label, val, min, max, step, page, dp, box, rb_group_or_combo_list, tooltip);
+*/
+
+
 #define PREF_REC_EXT_AUDIO "rec_ext_audio"
 #define PREF_AUDIO_OPTS "audio_opts"
-#define PREF_SEPWIN_STICKY "sepwin_sticky"
+#define PREF_SEPWIN_TYPE "sepwin_type"
 #define PREF_MT_EXIT_RENDER "mt_exit_render"
 
 // factories non-cpp
@@ -741,6 +788,8 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_CURRENT_AUTOTRANS "current_autotrans"
 #define PREF_DEF_AUTOTRANS "def_autotrans"  // readonly
 
+#define PREF_SCREEN_SCALE "screen_scale"
+
 ////////////////////// utf8 values
 
 #define PREF_OMC_MIDI_FNAME "omc_midi_fname"
@@ -754,6 +803,7 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_VID_SAVE_DIR "vid_save_dir"
 #define PREF_VID_LOAD_DIR "vid_load_dir"
 
+#define PREF_RECENT "recent"
 #define PREF_RECENT1 "recent1"
 #define PREF_RECENT2 "recent2"
 #define PREF_RECENT3 "recent3"
@@ -781,6 +831,9 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 
 #define PREF_MAX_DISP_VTRACKS "max_disp_vtracks"
 
+#define PREF_MAX_MSGS "max_text_messages"
+#define PREF_MSG_TEXTSIZE "msg_textsize"
+
 #define PREF_RTE_KEYS_VIRTUAL "rte_keys_virtual"
 
 #define PREF_JACK_OPTS "jack_opts"
@@ -844,6 +897,7 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_CONCAT_IMAGES "concat_images"
 #define PREF_SAVE_DIRECTORIES "save_directories"
 #define PREF_CONSERVE_SPACE "conserve_space"
+#define PREF_PUSH_AUDIO_TO_GENS "push_audio_to_gens"
 
 ////////// double values
 #define PREF_MT_DEF_FPS "mt_def_fps"
@@ -859,14 +913,15 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 // defaults (readonly)
 /////////////// string values
 #define PREF_SOX_COMMAND "sox_command"
-#define PREF_MPLAYER_AUDIO_COMMAND "mplayer_audio_command"
-#define PREF_MPLAYER2_AUDIO_COMMAND "mplayer2_audio_command"
 
-void pref_factory_bool(const char *prefidx, boolean newval);
-void pref_factory_int(const char *prefidx, int newval);
-void pref_factory_float(const char *prefidx, float newval);
-void pref_factory_bitmapped(const char *prefidx, int bitfield, boolean newval);
+boolean pref_factory_bool(const char *prefidx, boolean newval, boolean permanent);
+boolean pref_factory_string(const char *prefidx, const char *newval, boolean permanent);
+boolean pref_factory_int(const char *prefidx, int newval, boolean permanent);
+boolean pref_factory_float(const char *prefidx, float newval, boolean permanent);
+boolean pref_factory_bitmapped(const char *prefidx, int bitfield, boolean newval, boolean permanent);
+boolean pref_factory_string_choice(const char *prefidx, LiVESList *list, const char *strval, boolean permanent);
 
+boolean has_pref(const char *key);
 int get_pref(const char *key, char *val, int maxlen);
 int get_pref_from_file(const char *filename, const char *key, char *val, int maxlen);
 int get_pref_utf8(const char *key, char *val, int maxlen);
@@ -877,15 +932,15 @@ int get_int_pref(const char *key);
 LiVESList *get_list_pref(const char *key);
 boolean get_colour_pref(const char *key, lives_colRGBA64_t *lcol);
 boolean get_theme_colour_pref(const char *themefile, const char *key, lives_colRGBA64_t *lcol);
-void set_pref(const char *key, const char *value);
-void set_pref_utf8(const char *key, const char *value);
-void delete_pref(const char *key);
-void set_boolean_pref(const char *key, boolean value);
-void set_double_pref(const char *key, double value);
-void set_int_pref(const char *key, int value);
-void set_int64_pref(const char *key, int64_t value);
-void set_list_pref(const char *key, LiVESList *values);
-void set_colour_pref(const char *key, lives_colRGBA64_t *lcol);
+int set_pref(const char *key, const char *value);
+int set_pref_utf8(const char *key, const char *value);
+int delete_pref(const char *key);
+int set_boolean_pref(const char *key, boolean value);
+int set_double_pref(const char *key, double value);
+int set_int_pref(const char *key, int value);
+int set_int64_pref(const char *key, int64_t value);
+int set_list_pref(const char *key, LiVESList *values);
+int set_colour_pref(const char *key, lives_colRGBA64_t *lcol);
 void set_theme_pref(const char *themefile, const char *key, const char *value);
 void set_theme_colour_pref(const char *themefile, const char *key, lives_colRGBA64_t *lcol);
 boolean apply_prefs(boolean skip_warnings);
@@ -893,6 +948,8 @@ void save_future_prefs(void);
 
 void set_palette_prefs(void);
 
+void toggle_button_sets_pref(LiVESToggleButton *button, livespointer prefidx);
+
 // permissions
 
 #define LIVES_PERM_OSC_PORTS 1
