@@ -4,6 +4,8 @@
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
+#include "main.h"
+
 #include <errno.h>
 #include <sys/stat.h>
 
@@ -17,15 +19,12 @@
 #include "../libweed/weed-effects.h"
 #endif
 
-#include "main.h"
-
 #include "support.h"
 #include "rte_window.h"
 #include "effects.h"
 #include "paramwindow.h"
 #include "ce_thumbs.h"
 
-static LiVESWidget *rte_window_back = NULL;
 static int old_rte_keys_virtual = 0;
 
 static LiVESWidget **key_checks;
@@ -65,6 +64,14 @@ static char *empty_string = "";
 static void set_param_and_con_buttons(int key, int mode);
 static void check_clear_all_button(void);
 
+static boolean rte_window_is_hidden = TRUE;
+
+
+boolean rte_window_hidden(void) {
+  return rte_window_is_hidden;
+}
+
+
 void rte_window_set_interactive(boolean interactive) {
   register int i, j;
   int modes = rte_getmodespk();
@@ -142,9 +149,7 @@ boolean on_clear_all_clicked(LiVESButton *button, livespointer user_data) {
   ca_canc = FALSE;
 
   // prompt for "are you sure ?"
-  if (user_data != NULL) if (!do_warning_dialog_with_check_transient
-                               ((_("\n\nUnbind all effects from all keys/modes.\n\nAre you sure ?\n\n")),
-                                0, LIVES_WINDOW(rte_window))) {
+  if (user_data != NULL) if (!do_warning_dialog(_("\n\nUnbind all effects from all keys/modes.\n\nAre you sure ?\n\n"))) {
       ca_canc = TRUE;
       return FALSE;
     }
@@ -438,8 +443,7 @@ static boolean on_save_keymap_clicked(LiVESButton *button, livespointer user_dat
   int retval;
 
   if (button != NULL) {
-    if (!do_warning_dialog_with_check_transient
-        ((_("\n\nClick 'OK' to save this keymap as your default\n\n")), 0, LIVES_WINDOW(rte_window))) {
+    if (!do_warning_dialog(_("\n\nClick 'OK' to save this keymap as your default\n\n"))) {
       lives_free(keymap_file3);
       lives_free(keymap_file2);
       lives_free(keymap_file);
@@ -811,14 +815,14 @@ static boolean load_datacons(const char *fname, uint8_t **badkeymap) {
 
         memset(hashname + hlen, 0, 1);
 
-        if (omode < 0 || omode > maxmodes) is_valid = FALSE;
+        if (omode < 0 || omode >= maxmodes) is_valid = FALSE;
 
         if (is_valid) {
           // if we had bad/missing fx, adjust the omode value
           for (i = 0; i < omode; i++) omode -= badkeymap[okey][omode];
         }
 
-        if (omode < 0 || omode > maxmodes) is_valid = FALSE;
+        if (omode < 0 || omode >= maxmodes) is_valid = FALSE;
 
         if (is_valid) {
           int fidx = rte_keymode_get_filter_idx(okey + 1, omode);
@@ -903,14 +907,14 @@ static boolean load_datacons(const char *fname, uint8_t **badkeymap) {
 
             memset(hashname + hlen, 0, 1);
 
-            if (imode < 0 || imode > maxmodes) is_valid2 = FALSE;
+            if (imode < 0 || (ikey >= 0 && imode >= maxmodes)) is_valid2 = FALSE;
 
             if (is_valid2) {
               // if we had bad/missing fx, adjust the omode value
               for (k = 0; k < imode; k++) imode -= badkeymap[ikey][imode];
             }
 
-            if (imode < 0 || imode > maxmodes) is_valid2 = FALSE;
+            if (imode < 0 || (ikey >= 0 && imode >= maxmodes)) is_valid2 = FALSE;
 
             if (is_valid2) {
               int fidx = rte_keymode_get_filter_idx(ikey + 1, imode);
@@ -1004,14 +1008,14 @@ static boolean load_datacons(const char *fname, uint8_t **badkeymap) {
 
         memset(hashname + hlen, 0, 1);
 
-        if (omode < 0 || omode > maxmodes) is_valid = FALSE;
+        if (omode < 0 || omode >= maxmodes) is_valid = FALSE;
 
         if (is_valid) {
           // if we had bad/missing fx, adjust the omode value
           for (i = 0; i < omode; i++) omode -= badkeymap[okey][omode];
         }
 
-        if (omode < 0 || omode > maxmodes) is_valid = FALSE;
+        if (omode < 0 || omode >= maxmodes) is_valid = FALSE;
 
         if (is_valid) {
           int fidx = rte_keymode_get_filter_idx(okey + 1, omode);
@@ -1091,16 +1095,16 @@ static boolean load_datacons(const char *fname, uint8_t **badkeymap) {
 
             memset(hashname + hlen, 0, 1);
 
-            if (imode < 0 || imode > maxmodes) is_valid2 = FALSE;
+            if (imode < 0 || (ikey >= 0 && imode >= maxmodes)) is_valid2 = FALSE;
 
-            if (is_valid2) {
+            if (is_valid2 && ikey >= 0) {
               // if we had bad/missing fx, adjust the omode value
               for (k = 0; k < imode; k++) imode -= badkeymap[ikey][imode];
             }
 
-            if (imode < 0 || imode > maxmodes) is_valid2 = FALSE;
+            if (imode < 0 || (ikey >= 0 && imode >= maxmodes)) is_valid2 = FALSE;
 
-            if (is_valid2) {
+            if (is_valid2 && ikey >= 0) {
               int fidx = rte_keymode_get_filter_idx(ikey + 1, imode);
               if (fidx == -1) is_valid2 = FALSE;
               else {
@@ -1123,15 +1127,17 @@ static boolean load_datacons(const char *fname, uint8_t **badkeymap) {
               break;
             }
 
-            // check ipnum
-            filter = rte_keymode_get_filter(ikey + 1, imode);
-            niparams = weed_leaf_num_elements(filter, WEED_LEAF_IN_PARAMETER_TEMPLATES);
-            if (ipnum >= niparams) is_valid2 = FALSE;
-            else {
-              if (ipnum >= 0) {
-                iparams = weed_get_plantptr_array(filter, WEED_LEAF_IN_PARAMETER_TEMPLATES, &error);
-                if (weed_plant_has_leaf(iparams[ipnum], WEED_LEAF_HOST_INTERNAL_CONNECTION)) is_valid2 = FALSE;
-                lives_free(iparams);
+            if (ikey >= 0) {
+              // check ipnum
+              filter = rte_keymode_get_filter(ikey + 1, imode);
+              niparams = weed_leaf_num_elements(filter, WEED_LEAF_IN_PARAMETER_TEMPLATES);
+              if (ipnum >= niparams) is_valid2 = FALSE;
+              else {
+                if (ipnum >= 0) {
+                  iparams = weed_get_plantptr_array(filter, WEED_LEAF_IN_PARAMETER_TEMPLATES, &error);
+                  if (weed_plant_has_leaf(iparams[ipnum], WEED_LEAF_HOST_INTERNAL_CONNECTION)) is_valid2 = FALSE;
+                  lives_free(iparams);
+                }
               }
             }
 
@@ -1176,7 +1182,7 @@ static void set_param_and_con_buttons(int key, int mode) {
     if (num_in_params(filter, TRUE, TRUE) > 0) lives_widget_set_sensitive(param_buttons[idx], TRUE);
     else lives_widget_set_sensitive(param_buttons[idx], FALSE);
     lives_widget_set_sensitive(combos[idx], TRUE);
-    if (mode < modes - 1) lives_widget_set_sensitive(combos[idx + 1], TRUE);
+    if (combos[idx + 1] != NULL && mode < modes - 1) lives_widget_set_sensitive(combos[idx + 1], TRUE);
   } else {
     lives_widget_set_sensitive(conx_buttons[idx], FALSE);
     lives_widget_set_sensitive(param_buttons[idx], FALSE);
@@ -1479,7 +1485,7 @@ boolean on_load_keymap_clicked(LiVESButton *button, livespointer user_data) {
       int idx = (key - 1) * modes + mode;
       int fx_idx = rte_keymode_get_filter_idx(key, mode);
 
-      rtew_combo_set_text(key, mode, (tmp = rte_keymode_get_filter_name(key, mode)));
+      rtew_combo_set_text(key - 1, mode, (tmp = rte_keymode_get_filter_name(key, mode)));
       lives_free(tmp);
 
       if (fx_idx != -1) {
@@ -1520,8 +1526,7 @@ boolean on_load_keymap_clicked(LiVESButton *button, livespointer user_data) {
 
     if (mainw->is_ready) {
       check_clear_all_button();
-      if (notfound) do_warning_dialog_with_check_transient(_("\n\nSome effects could not be located.\n\n"),
-            0, LIVES_WINDOW(rte_window));
+      if (notfound) do_warning_dialog(_("\n\nSome effects could not be located.\n\n"));
     } else load_rte_defs(); // file errors shown inside
 
   }
@@ -1623,6 +1628,7 @@ void on_rte_info_clicked(LiVESButton *button, livespointer user_data) {
 
   vbox = lives_dialog_get_content_area(LIVES_DIALOG(dialog));
 
+  widget_opts.justify = LIVES_JUSTIFY_CENTER;
   label = lives_standard_label_new((tmp = lives_strdup_printf(_("Effect name: %s"), filter_name)));
   lives_free(tmp);
   lives_box_pack_start(LIVES_BOX(vbox), label, TRUE, FALSE, widget_opts.packing_height);
@@ -1684,20 +1690,25 @@ void on_rte_info_clicked(LiVESButton *button, livespointer user_data) {
     lives_box_pack_start(LIVES_BOX(vbox), label, TRUE, FALSE, widget_opts.packing_height);
   }
 
-  ok_button = lives_standard_button_new_from_stock(LIVES_STOCK_OK, NULL);
-  lives_dialog_add_action_widget(LIVES_DIALOG(dialog), ok_button, LIVES_RESPONSE_OK);
+  widget_opts.justify = LIVES_JUSTIFY_DEFAULT;
 
   abox = lives_dialog_get_action_area(LIVES_DIALOG(dialog));
+#if GTK_CHECK_VERSION(3, 0, 0)
+  if (LIVES_IS_BOX(abox)) add_spring_to_box(LIVES_BOX(abox), 0);
+#endif
+
+  widget_opts.expand = LIVES_EXPAND_DEFAULT_HEIGHT | LIVES_EXPAND_EXTRA_WIDTH;
+  ok_button = lives_dialog_add_button_from_stock(LIVES_DIALOG(dialog), LIVES_STOCK_OK, NULL,
+              LIVES_RESPONSE_OK);
+  widget_opts.expand = LIVES_EXPAND_DEFAULT;
+
 #if !GTK_CHECK_VERSION(3, 0, 0)
   lives_button_box_set_layout(LIVES_BUTTON_BOX(abox), LIVES_BUTTONBOX_CENTER);
 #else
-  if (LIVES_IS_BOX(abox)) add_fill_to_box(LIVES_BOX(abox));
+  if (LIVES_IS_BOX(abox)) add_spring_to_box(LIVES_BOX(abox), 0);
 #endif
 
-  lives_widget_set_can_focus_and_default(ok_button);
-  lives_widget_grab_default_special(ok_button);
-
-  lives_widget_set_size_request(ok_button, DEF_BUTTON_WIDTH * 4, -1);
+  lives_button_grab_default_special(ok_button);
 
   lives_signal_connect(LIVES_GUI_OBJECT(ok_button), LIVES_WIDGET_CLICKED_SIGNAL,
                        LIVES_GUI_CALLBACK(lives_general_button_clicked),
@@ -1791,7 +1802,6 @@ static void on_datacon_clicked(LiVESButton *button, livespointer user_data) {
   int mode = idx - key * modes;
 
   //if (datacon_dialog!=NULL) on_datacon_cancel_clicked(NULL,NULL);
-
   datacon_dialog = make_datacon_window(key, mode);
 }
 
@@ -1806,21 +1816,16 @@ static void on_params_clicked(LiVESButton *button, livespointer user_data) {
   lives_rfx_t *rfx;
 
   if ((inst = rte_keymode_get_instance(key + 1, mode)) == NULL) {
+    // create a new detached instance for the dialog
     weed_plant_t *filter = rte_keymode_get_filter(key + 1, mode);
     if (filter == NULL) return;
     inst = weed_instance_from_filter(filter);
+    weed_set_boolean_value(inst, WEED_LEAF_HOST_NORECORD, WEED_TRUE);
 
     // do some fiddly stuff to show the key defs.
     weed_reinit_effect(inst, TRUE);
     apply_key_defaults(inst, key, mode);
     weed_reinit_effect(inst, TRUE);
-  } else {
-    int error;
-    weed_plant_t *ninst = inst;
-    do {
-      weed_instance_ref(ninst);
-    } while (weed_plant_has_leaf(ninst, WEED_LEAF_HOST_NEXT_INSTANCE) &&
-             (ninst = weed_get_plantptr_value(ninst, WEED_LEAF_HOST_NEXT_INSTANCE, &error)) != NULL);
   }
 
   if (fx_dialog[1] != NULL) {
@@ -1829,11 +1834,15 @@ static void on_params_clicked(LiVESButton *button, livespointer user_data) {
     on_paramwindow_cancel_clicked2(NULL, rfx);
   }
 
-  rfx = weed_to_rfx(inst, FALSE);
+  rfx = weed_to_rfx(inst, FALSE); // rfx inherits the refcount
   rfx->min_frames = -1;
   keyw = key;
   modew = mode;
+
+  widget_opts.non_modal = TRUE;
   on_fx_pre_activate(rfx, 1, NULL);
+  widget_opts.non_modal = FALSE;
+
 
   // record the key so we know whose parameters to record later
   weed_set_int_value((weed_plant_t *)rfx->source, WEED_LEAF_HOST_KEY, key);
@@ -1845,11 +1854,14 @@ static void on_params_clicked(LiVESButton *button, livespointer user_data) {
 
 
 static boolean on_rtew_delete_event(LiVESWidget *widget, LiVESXEventDelete *event, livespointer user_data) {
+  old_rte_keys_virtual = prefs->rte_keys_virtual;
   if (user_data == NULL) {
-    rte_window_back = rte_window;
-    old_rte_keys_virtual = prefs->rte_keys_virtual;
+    // first time around we come here, and just hide the window
     lives_widget_hide(rte_window);
+    rte_window_is_hidden = TRUE;
   } else {
+    // when we reshow it we check if the number of fx keys has changed. If so we come back to this branch
+    // and recreate the window from scratch
     lives_list_free_all(&hash_list);
     lives_list_free_all(&name_list);
     lives_list_free_all(&name_type_list);
@@ -1869,20 +1881,18 @@ static boolean on_rtew_delete_event(LiVESWidget *widget, LiVESXEventDelete *even
     lives_free(conx_buttons);
     lives_free(clear_buttons);
   }
-  rte_window = NULL;
   return FALSE;
 }
 
 
-static void on_rtew_ok_clicked(LiVESButton *button, livespointer user_data) {
+static boolean on_rtew_ok_clicked(LiVESButton *button, livespointer user_data) {
   on_rtew_delete_event(NULL, NULL, NULL);
+  return TRUE;
 }
 
 
-static void do_mix_error(void) {
-  do_error_dialog_with_check_transient(
-    _("\n\nThis version of LiVES does not allowing mixing of generators and non-generators on the same key.\n\n"),
-    FALSE, 0, LIVES_WINDOW(rte_window));
+LIVES_LOCAL_INLINE void do_mix_error(void) {
+  do_error_dialog(_("\n\nThis version of LiVES does not allowing mixing of generators and non-generators on the same key.\n\n"));
   return;
 }
 
@@ -1979,9 +1989,9 @@ void fx_changed(LiVESCombo *combo, livespointer user_data) {
   if (mainw->ce_thumbs) ce_thumbs_reset_combos();
 }
 
+static LiVESTreeStore *tstore = NULL;
 
 static LiVESTreeModel *rte_window_fx_model(void) {
-  LiVESTreeStore *tstore;
 
   LiVESTreeIter iter1, iter2;
 
@@ -1989,28 +1999,38 @@ static LiVESTreeModel *rte_window_fx_model(void) {
   int fx_idx = 0;
 
   LiVESList *list = name_type_list;
+  LiVESList *pname_list = name_list;
+  LiVESList *phash_list = hash_list;
 
   int error;
 
   char *pkg = NULL, *pkgstring, *fxname;
 
-  tstore = lives_tree_store_new(NUM_COLUMNS, LIVES_COL_TYPE_STRING, LIVES_COL_TYPE_STRING, LIVES_COL_TYPE_STRING);
+  if (tstore != NULL) return (LiVESTreeModel *)tstore;
 
+  tstore = lives_tree_store_new(NUM_COLUMNS, LIVES_COL_TYPE_STRING, LIVES_COL_TYPE_STRING, LIVES_COL_TYPE_STRING);
+#ifdef GUI_GTK
+  // supposedly speeds things up a bit...
+  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(tstore), GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+                                       GTK_SORT_ASCENDING);
+#endif
   while (list != NULL) {
-    weed_plant_t *filter = get_weed_filter(weed_get_idx_for_hashname((char *)lives_list_nth_data(hash_list, fx_idx), TRUE));
+    weed_plant_t *filter = get_weed_filter(weed_get_idx_for_hashname((char *)phash_list->data, TRUE));
     int filter_flags = weed_get_int_value(filter, WEED_LEAF_FLAGS, &error);
     if ((weed_plant_has_leaf(filter, WEED_LEAF_PLUGIN_UNSTABLE) && weed_get_boolean_value(filter, WEED_LEAF_PLUGIN_UNSTABLE, &error) ==
          WEED_TRUE && !prefs->unstable_fx) || ((enabled_in_channels(filter, FALSE) > 1 &&
              !has_video_chans_in(filter, FALSE)) ||
              (weed_plant_has_leaf(filter, WEED_LEAF_HOST_MENU_HIDE) &&
               weed_get_boolean_value(filter, WEED_LEAF_HOST_MENU_HIDE, &error) == WEED_TRUE)
-             || (filter_flags & WEED_FILTER_IS_CONVERTER))) {
+             || (filter_flags & WEED_FILTER_IS_CONVERTER)) || enabled_in_channels(filter, TRUE) == 1000000) {
       list = list->next;
       fx_idx++;
-      continue; // skip audio transitions and hidden entries
+      pname_list = pname_list->next;
+      phash_list = phash_list->next;
+      continue; // skip audio transitions, compositors and hidden entries
     }
 
-    fxname = lives_strdup((char *)lives_list_nth_data(name_list, fx_idx));
+    fxname = lives_strdup((const char *)pname_list->data);
 
     if ((pkgstring = strstr(fxname, ": ")) != NULL) {
       // package effect
@@ -2026,7 +2046,7 @@ static LiVESTreeModel *rte_window_fx_model(void) {
         memset(pkgstring, 0, 1);
         /* TRANSLATORS: example " - LADSPA plugins -" */
         pkgstring = lives_strdup_printf(_(" - %s plugins -"), pkg);
-        lives_tree_store_append(tstore, &iter1, NULL);
+        lives_tree_store_prepend(tstore, &iter1, NULL);
         lives_tree_store_set(tstore, &iter1, NAME_TYPE_COLUMN, pkgstring, NAME_COLUMN, fxname,
                              HASH_COLUMN, lives_list_nth_data(hash_list, fx_idx), -1);
         lives_free(pkgstring);
@@ -2045,6 +2065,8 @@ static LiVESTreeModel *rte_window_fx_model(void) {
 
     list = list->next;
     fx_idx++;
+    pname_list = pname_list->next;
+    phash_list = phash_list->next;
   }
 
   lives_freep((void **)&pkg);
@@ -2054,7 +2076,7 @@ static LiVESTreeModel *rte_window_fx_model(void) {
 
 
 LiVESWidget *create_rte_window(void) {
-  LiVESWidget *rte_window;
+  LiVESWidget *irte_window = rte_window;
   LiVESWidget *table;
   LiVESWidget *hbox;
   LiVESWidget *hbox2;
@@ -2090,14 +2112,17 @@ LiVESWidget *create_rte_window(void) {
 
   lives_set_cursor_style(LIVES_CURSOR_BUSY, NULL);
   lives_widget_context_update();
+  mainw->no_context_update = TRUE;
 
   winsize_h = GUI_SCREEN_WIDTH - SCR_WIDTH_SAFETY;
   winsize_v = GUI_SCREEN_HEIGHT - SCR_HEIGHT_SAFETY;
 
-  if (rte_window_back != NULL) {
-    rte_window = rte_window_back;
-    rte_window_back = NULL;
-    if (prefs->rte_keys_virtual != old_rte_keys_virtual) return refresh_rte_window();
+  if (irte_window != NULL) {
+    if (prefs->rte_keys_virtual != old_rte_keys_virtual) {
+      // number of fx keys changed, rebuild the window
+      mainw->no_context_update = FALSE;
+      return refresh_rte_window();
+    }
     goto rte_window_ready;
   }
 
@@ -2117,13 +2142,15 @@ LiVESWidget *create_rte_window(void) {
   gr_fns = (ulong *)lives_malloc((prefs->rte_keys_virtual) * sizeof(ulong));
   mode_ra_fns = (ulong *)lives_malloc((prefs->rte_keys_virtual) * modes * sizeof(ulong));
 
-  rte_window = lives_window_new(LIVES_WINDOW_TOPLEVEL);
+  irte_window = lives_window_new(LIVES_WINDOW_TOPLEVEL);
+  lives_window_set_transient_for(LIVES_WINDOW(irte_window), LIVES_WINDOW(LIVES_MAIN_WINDOW_WIDGET));
+
   if (palette->style & STYLE_1) {
-    lives_widget_set_bg_color(rte_window, LIVES_WIDGET_STATE_NORMAL, &palette->menu_and_bars);
-    lives_widget_set_text_color(rte_window, LIVES_WIDGET_STATE_NORMAL, &palette->menu_and_bars_fore);
+    lives_widget_set_bg_color(irte_window, LIVES_WIDGET_STATE_NORMAL, &palette->menu_and_bars);
+    lives_widget_set_text_color(irte_window, LIVES_WIDGET_STATE_NORMAL, &palette->menu_and_bars_fore);
   }
-  lives_window_set_title(LIVES_WINDOW(rte_window), _("Real Time Effect Mapping"));
-  lives_window_add_accel_group(LIVES_WINDOW(rte_window), mainw->accel_group);
+  lives_window_set_title(LIVES_WINDOW(irte_window), _("Real Time Effect Mapping"));
+  lives_window_add_accel_group(LIVES_WINDOW(irte_window), mainw->accel_group);
 
   table = lives_table_new(prefs->rte_keys_virtual, modes + 1, FALSE);
 
@@ -2133,6 +2160,7 @@ LiVESWidget *create_rte_window(void) {
   // dummy button for "no grab", we dont show this...there is a button instead
   dummy_radio = lives_radio_button_new(grab_group);
   grab_group = lives_radio_button_get_group(LIVES_RADIO_BUTTON(dummy_radio));
+  lives_widget_set_no_show_all(dummy_radio, TRUE);
 
   name_list = weed_get_all_names(FX_LIST_NAME);
   name_type_list = weed_get_all_names(FX_LIST_NAME_AND_TYPE);
@@ -2142,7 +2170,7 @@ LiVESWidget *create_rte_window(void) {
 
   for (i = 0; i < prefs->rte_keys_virtual * modes; i++) {
     // create combo entry model
-    combos[i] = lives_combo_new_with_model(model);
+    combos[i] = NULL;
   }
 
   for (i = 0; i < prefs->rte_keys_virtual; i++) {
@@ -2184,8 +2212,6 @@ LiVESWidget *create_rte_window(void) {
 
     mode_group = NULL;
 
-    clear_all_button = lives_standard_button_new_from_stock(LIVES_STOCK_CLEAR, _("_Clear all effects"));
-
     for (j = 0; j < modes; j++) {
       idx = i * modes + j;
       hbox = lives_hbox_new(FALSE, 0);
@@ -2216,18 +2242,17 @@ LiVESWidget *create_rte_window(void) {
       lives_container_set_border_width(LIVES_CONTAINER(vbox), widget_opts.border_width);
 
       hbox = lives_hbox_new(FALSE, 0);
+
       lives_box_pack_start(LIVES_BOX(vbox), hbox, FALSE, FALSE, widget_opts.packing_height);
 
       nlabels[idx] = lives_standard_label_new(_("Effect name:"));
 
       lives_box_pack_start(LIVES_BOX(hbox), nlabels[idx], FALSE, FALSE, widget_opts.packing_width);
 
-      combo = combos[idx];
-
+      combos[idx] = combo = lives_standard_combo_new_with_model(model, LIVES_BOX(hbox));
       lives_combo_set_entry_text_column(LIVES_COMBO(combo), NAME_TYPE_COLUMN);
 
       lives_widget_object_set_data(LIVES_WIDGET_OBJECT(combo), "hashname", empty_string);
-      lives_box_pack_start(LIVES_BOX(hbox), combo, TRUE, TRUE, widget_opts.packing_width);
       lives_box_pack_end(LIVES_BOX(hbox), clear_buttons[idx], FALSE, FALSE, widget_opts.packing_width);
       lives_box_pack_end(LIVES_BOX(hbox), info_buttons[idx], FALSE, FALSE, widget_opts.packing_width);
 
@@ -2272,7 +2297,7 @@ LiVESWidget *create_rte_window(void) {
   lives_box_pack_start(LIVES_BOX(top_vbox), dummy_radio, FALSE, FALSE, 0);
   lives_box_pack_start(LIVES_BOX(top_vbox), scrolledwindow, TRUE, TRUE, widget_opts.packing_height);
 
-  lives_container_add(LIVES_CONTAINER(rte_window), top_vbox);
+  lives_container_add(LIVES_CONTAINER(irte_window), top_vbox);
 
   hbuttonbox = lives_hbutton_box_new();
   label = add_fill_to_box(LIVES_BOX(hbuttonbox));
@@ -2283,23 +2308,25 @@ LiVESWidget *create_rte_window(void) {
 
   lives_box_pack_start(LIVES_BOX(top_vbox), hbuttonbox, FALSE, TRUE, widget_opts.packing_height * 2);
 
-  lives_container_add(LIVES_CONTAINER(hbuttonbox), clear_all_button);
-  lives_widget_set_can_focus_and_default(clear_all_button);
+  clear_all_button = lives_dialog_add_button_from_stock(NULL, LIVES_STOCK_CLEAR, _("_Clear all effects"),
+                     LIVES_RESPONSE_RESET);
 
-  save_keymap_button = lives_standard_button_new_from_stock(LIVES_STOCK_SAVE_AS, _("_Save as default keymap"));
+  lives_container_add(LIVES_CONTAINER(hbuttonbox), clear_all_button);
+
+  save_keymap_button = lives_dialog_add_button_from_stock(NULL, LIVES_STOCK_SAVE_AS, _("_Save as default keymap"),
+                       LIVES_RESPONSE_ACCEPT);
 
   lives_container_add(LIVES_CONTAINER(hbuttonbox), save_keymap_button);
-  lives_widget_set_can_focus_and_default(save_keymap_button);
 
-  load_keymap_button = lives_standard_button_new_from_stock(LIVES_STOCK_OPEN, _("_Load default keymap"));
+  load_keymap_button = lives_dialog_add_button_from_stock(NULL, LIVES_STOCK_OPEN, _("_Load default keymap"),
+                       LIVES_RESPONSE_BROWSE);
 
   lives_container_add(LIVES_CONTAINER(hbuttonbox), load_keymap_button);
-  lives_widget_set_can_focus_and_default(load_keymap_button);
 
-  ok_button = lives_standard_button_new_from_stock(LIVES_STOCK_CLOSE, _("Close _window"));
+  ok_button = lives_dialog_add_button_from_stock(NULL, LIVES_STOCK_CLOSE, _("Close _window"),
+              LIVES_RESPONSE_OK);
 
   lives_container_add(LIVES_CONTAINER(hbuttonbox), ok_button);
-  lives_widget_set_can_focus_and_default(ok_button);
 
   label = add_fill_to_box(LIVES_BOX(hbuttonbox));
 
@@ -2313,12 +2340,12 @@ LiVESWidget *create_rte_window(void) {
   lives_button_box_set_button_width(LIVES_BUTTON_BOX(hbuttonbox), ok_button, DEF_BUTTON_WIDTH);
 
   rtew_accel_group = LIVES_ACCEL_GROUP(lives_accel_group_new());
-  lives_window_add_accel_group(LIVES_WINDOW(rte_window), rtew_accel_group);
+  lives_window_add_accel_group(LIVES_WINDOW(irte_window), rtew_accel_group);
 
   lives_widget_add_accelerator(ok_button, LIVES_WIDGET_CLICKED_SIGNAL, rtew_accel_group,
                                LIVES_KEY_Escape, (LiVESXModifierType)0, (LiVESAccelFlags)0);
 
-  lives_signal_connect(LIVES_GUI_OBJECT(rte_window), LIVES_WIDGET_DELETE_EVENT,
+  lives_signal_connect(LIVES_GUI_OBJECT(irte_window), LIVES_WIDGET_DELETE_EVENT,
                        LIVES_GUI_CALLBACK(on_rtew_ok_clicked),
                        NULL);
 
@@ -2339,20 +2366,22 @@ LiVESWidget *create_rte_window(void) {
                        LIVES_INT_TO_POINTER(1));
 
 rte_window_ready:
+  // TODO: ignore button clicks until window is fully shown
 
-  lives_widget_show_all(rte_window);
-  lives_widget_hide(dummy_radio);
+  rte_window_is_hidden = FALSE;
 
-  if (prefs->gui_monitor != 0) {
-    lives_window_center(LIVES_WINDOW(rte_window));
-  }
+  lives_widget_show_all(irte_window);
 
   if (prefs->open_maximised) {
-    lives_window_maximize(LIVES_WINDOW(rte_window));
+    lives_window_maximize(LIVES_WINDOW(irte_window));
   }
+
+  lives_widget_show_now(irte_window);
+
   lives_set_cursor_style(LIVES_CURSOR_NORMAL, NULL);
-  lives_set_cursor_style(LIVES_CURSOR_NORMAL, rte_window);
-  return rte_window;
+  lives_set_cursor_style(LIVES_CURSOR_NORMAL, irte_window);
+  mainw->no_context_update = FALSE;
+  return irte_window;
 }
 
 
@@ -2361,7 +2390,7 @@ LiVESWidget *refresh_rte_window(void) {
     lives_set_cursor_style(LIVES_CURSOR_BUSY, NULL);
     lives_set_cursor_style(LIVES_CURSOR_BUSY, rte_window);
     lives_widget_context_update();
-    on_rtew_delete_event(NULL, NULL, LIVES_INT_TO_POINTER(1));
+    on_rtew_delete_event(NULL, NULL, NULL);
     lives_widget_destroy(rte_window);
     rte_window = create_rte_window();
     rte_window_set_interactive(mainw->interactive);
@@ -2371,14 +2400,12 @@ LiVESWidget *refresh_rte_window(void) {
 
 
 void on_assign_rte_keys_activate(LiVESMenuItem *menuitem, livespointer user_data) {
-  if (rte_window != NULL) {
-    on_rtew_ok_clicked(LIVES_BUTTON(dummy_radio), user_data);
-    return;
+  if (rte_window != NULL && !rte_window_is_hidden) on_rtew_delete_event(NULL, NULL, NULL);
+  else {
+    rte_window = create_rte_window();
+    rte_window_set_interactive(mainw->interactive);
+    lives_widget_show(rte_window);
   }
-
-  rte_window = create_rte_window();
-  rte_window_set_interactive(mainw->interactive);
-  lives_widget_show(rte_window);
 }
 
 
@@ -2413,6 +2440,7 @@ void rtew_set_mode_radio(int key, int mode) {
 
 
 void redraw_pwindow(int key, int mode) {
+  // called when the source inst is reinited / inited / deinited
   LiVESList *child_list;
   lives_rfx_t *rfx;
 
@@ -2430,9 +2458,13 @@ void redraw_pwindow(int key, int mode) {
     }
     if (rfx->is_template || (key == keyw && mode == modew)) {
       // rip out the contents
+      // TODO !!!:: if we got here by way of a widget callback which caused a reinit, we can crash
+      // since gtk+ will try to continue handling the callback with an invalid widget
+      // so, we should note the widget that received the signal and the signal type
+      // then stop emmission of the signal before destroying it
+
       content_area = lives_dialog_get_content_area(LIVES_DIALOG(fx_dialog[1]));
       child_list = lives_container_get_children(LIVES_CONTAINER(content_area));
-      // remove focus from any widget we are ripping out
       for (i = 0; i < lives_list_length(child_list); i++) {
         LiVESWidget *widget = (LiVESWidget *)lives_list_nth_data(child_list, i);
         if (lives_widget_is_ancestor(LIVES_WIDGET(button), widget)) continue;
@@ -2458,7 +2490,7 @@ void restore_pwindow(lives_rfx_t *rfx) {
 void update_pwindow(int key, int i, LiVESList *list) {
   // called only from weed_set_blend_factor() and from setting param in ce_thumbs
 
-  const weed_plant_t *inst;
+  weed_plant_t *inst;
   lives_rfx_t *rfx;
   int keyw, modew;
 
@@ -2467,6 +2499,7 @@ void update_pwindow(int key, int i, LiVESList *list) {
     modew = LIVES_POINTER_TO_INT(lives_widget_object_get_data(LIVES_WIDGET_OBJECT(fx_dialog[1]), "mode"));
     if (key == keyw) {
       if ((inst = rte_keymode_get_instance(key + 1, modew)) == NULL) return;
+      weed_instance_unref(inst);
       rfx = (lives_rfx_t *)lives_widget_object_get_data(LIVES_WIDGET_OBJECT(fx_dialog[1]), "rfx");
       mainw->block_param_updates = TRUE;
       set_param_from_list(list, &rfx->params[i], 0, TRUE, TRUE);
@@ -2475,6 +2508,7 @@ void update_pwindow(int key, int i, LiVESList *list) {
   }
 }
 
+
 void rte_set_defs_activate(LiVESMenuItem *menuitem, livespointer user_data) {
   int idx = LIVES_POINTER_TO_INT(user_data);
   weed_plant_t *filter = get_weed_filter(idx);
@@ -2703,7 +2737,6 @@ void load_default_keymap(void) {
       }
 
       lives_cp(keymap_template, keymap_file);
-
     }
     if (!lives_file_test(keymap_file, LIVES_FILE_TEST_EXISTS)) {
       // give up
