@@ -752,7 +752,7 @@ void weed_layer_set_from_lives2lives(weed_plant_t *layer, int clip, lives_vstrea
         // this seems to help smoothing when recording, however I have only tested it on one machine
         // where frames were being generated and streamed and then received
         // - needs testing in other situations
-        currticks = lives_get_current_ticks(mainw->origsecs, mainw->origusecs);
+        currticks = lives_get_current_ticks();
 
         if (mainw->record && !mainw->record_paused) {
           if (has_last_delta_ticks && (abs64(currticks - lstream->timecode)) < last_delta_ticks) {
@@ -1034,9 +1034,9 @@ void on_open_lives2lives_activate(LiVESMenuItem *menuitem, livespointer user_dat
                                  lives_entry_get_text(LIVES_ENTRY(pandh->entry3)), lives_entry_get_text(LIVES_ENTRY(pandh->entry4)));
     } else host = lives_strdup("INADDR_ANY");
     port = lives_spin_button_get_value_as_int(LIVES_SPIN_BUTTON(pandh->port_spin));
+    lives_widget_destroy(pandh->dialog);
   }
 
-  lives_widget_destroy(pandh->dialog);
   lives_free(pandh);
 
   lives_widget_context_update();
@@ -1081,10 +1081,6 @@ lives_pandh_w *create_pandh_dialog(int type) {
 
   pandhw->dialog = lives_standard_dialog_new(_("Receive LiVES Stream"), TRUE, -1, -1);
 
-  if (prefs->show_gui) {
-    lives_window_set_transient_for(LIVES_WINDOW(pandhw->dialog), LIVES_WINDOW(mainw->LiVES));
-  }
-
   dialog_vbox = lives_dialog_get_content_area(LIVES_DIALOG(pandhw->dialog));
 
   label = lives_standard_label_new(_("You can receive streams from another copy of LiVES."));
