@@ -1,6 +1,6 @@
 // widget-helper-gtk.h
 // LiVES
-// (c) G. Finch 2012 - 2017 <salsaman@gmail.com>
+// (c) G. Finch 2012 - 2019 <salsaman+lives@gmail.com>
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -11,17 +11,40 @@
 
 #ifdef GUI_GTK
 
-#define GTK_RADIO_MENU_BUG // a bug where gtk_radio_menu_item_set_active() does not update visually
-#define GTK_SUBMENU_SENS_BUG // a bug where setting a menuitem insensitive fails if it has a submenu
-#define GTK_TEXT_VIEW_DRAW_BUG // a bug where textview crashes if too much text in it
+#ifdef ENABLE_GIW_3
+#include "giw/giwtimeline.h"
+#endif
 
-#if !GTK_CHECK_VERSION(3, 18, 9) // fixed version 
-#define GTK_TEXT_VIEW_CSS_BUG // a bug where named textviews cannot be set by CSS
+#ifdef ENABLE_GIW
+#include "giw/giwvslider.h"
+#include "giw/giwled.h"
+#endif
+
+
+// a bug where gtk_radio_menu_item_set_active() does not update visually
+// workaround: use check menuitems and update manually
+#define GTK_RADIO_MENU_BUG
+
+// a bug where setting a menuitem insensitive fails if it has a submenu
+// workaround: (?) unparent submenu, change its state, reparent it
+#define GTK_SUBMENU_SENS_BUG
+
+// a bug where textview crashes if too much text in it (maybe not a bug, was missing expose function ?)
+#define GTK_TEXT_VIEW_DRAW_BUG
+
+#if !GTK_CHECK_VERSION(3, 18, 9) // fixed version
+// a bug where named textviews cannot be set by CSS
+#define GTK_TEXT_VIEW_CSS_BUG
+#endif
+
+#if GTK_CHECK_VERSION(3, 0, 0)
+// a bug where CSS selectors (e.g. "entry" or ":insensitive") do nothing
+#define GTK_CSS_SELECTORS_BUG
 #endif
 
 #ifndef IS_MINGW
 typedef gboolean                          boolean;
-#endif
+#else
 
 #ifdef GDK_WINDOWING_WIN32
 #include <gdk/gdkwin32.h>
@@ -30,8 +53,12 @@ typedef gboolean                          boolean;
 #define GDK_IS_WIN32_DISPLAY(display) (TRUE)
 #endif
 
+typedef uint8_t                           boolean;
+
 #endif //GDK_WINDOWING_WIN32
 
+#endif // IS_MINGW
+
 #ifdef GDK_WINDOWING_X11
 
 // needed for GDK_WINDOW_XID - for fileselector preview
@@ -57,47 +84,6 @@ typedef gboolean                          boolean;
 
 #endif // GUI_GTK
 
-#ifdef PAINTER_CAIRO
-
-#ifndef GUI_GTK
-#include <cairo/cairo.h>
-#endif
-
-typedef cairo_t lives_painter_t;
-typedef cairo_surface_t lives_painter_surface_t;
-
-boolean lives_painter_surface_destroy(lives_painter_surface_t *);
-
-typedef cairo_format_t lives_painter_format_t;
-
-#define LIVES_PAINTER_FORMAT_A1   CAIRO_FORMAT_A1
-#define LIVES_PAINTER_FORMAT_A8   CAIRO_FORMAT_A8
-#define LIVES_PAINTER_FORMAT_ARGB32 CAIRO_FORMAT_ARGB32
-
-typedef cairo_content_t lives_painter_content_t; // eg. color, alpha, color+alpha
-
-#define LIVES_PAINTER_CONTENT_COLOR CAIRO_CONTENT_COLOR
-
-typedef cairo_operator_t lives_painter_operator_t;
-
-#define LIVES_PAINTER_OPERATOR_UNKNOWN CAIRO_OPERATOR_OVER
-#define LIVES_PAINTER_OPERATOR_DEFAULT CAIRO_OPERATOR_OVER
-
-#define LIVES_PAINTER_OPERATOR_DEST_OUT CAIRO_OPERATOR_DEST_OUT
-#if CAIRO_VERSION < CAIRO_VERSION_ENCODE(1, 10, 0)
-#define LIVES_PAINTER_OPERATOR_DIFFERENCE CAIRO_OPERATOR_OVER
-#define LIVES_PAINTER_OPERATOR_OVERLAY CAIRO_OPERATOR_OVER
-#else
-#define LIVES_PAINTER_OPERATOR_DIFFERENCE CAIRO_OPERATOR_DIFFERENCE
-#define LIVES_PAINTER_OPERATOR_OVERLAY CAIRO_OPERATOR_OVERLAY
-#endif
-
-typedef cairo_fill_rule_t lives_painter_fill_rule_t;
-
-#define LIVES_PAINTER_FILL_RULE_WINDING  CAIRO_FILL_RULE_WINDING
-#define LIVES_PAINTER_FILL_RULE_EVEN_ODD CAIRO_FILL_RULE_EVEN_ODD
-
-#endif
 
 #ifdef GUI_GTK
 
@@ -168,6 +154,7 @@ typedef gint(*LiVESCompareFunc)(gconstpointer a, gconstpointer b);
 #define LIVES_MAXUINT32 G_MAXUINT32
 #define LIVES_MAXSIZE G_MAXSIZE
 #define LIVES_MAXFLOAT G_MAXFLOAT
+#define LIVES_MAXDOUBLE G_MAXDOUBLE
 
 #define LIVES_IS_RTL (gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL)
 
@@ -181,6 +168,7 @@ typedef gint(*LiVESCompareFunc)(gconstpointer a, gconstpointer b);
 #define lives_snprintf(a, b, ...) g_snprintf(a, b, __VA_ARGS__)
 #define lives_strsplit(a, b, c) g_strsplit(a, b, c)
 #define lives_strfreev(a) g_strfreev(a)
+#define lives_strdup(a) g_strdup(a)
 #define lives_ascii_strcasecmp(a, b) g_ascii_strcasecmp(a, b)
 #define lives_ascii_strncasecmp(a, b, c) g_ascii_strncasecmp(a, b, c)
 #define lives_strconcat(a, ...) g_strconcat(a, __VA_ARGS__)
@@ -192,12 +180,13 @@ typedef gint(*LiVESCompareFunc)(gconstpointer a, gconstpointer b);
 #define LIVES_NORMALIZE_DEFAULT G_NORMALIZE_DEFAULT
 
 #define lives_utf8_normalize(a, b, c) g_utf8_normalize(a, b, c)
+#define lives_utf8_collate(a, b) g_utf8_collate(a, b)
+#define lives_utf8_casefold(a, b) g_utf8_casefold(a, b)
 
 #define lives_list_nth_data(list, i) g_list_nth_data(list, i)
 #define lives_list_nth(list, i) g_list_nth(list, i)
 #define lives_list_length(list) g_list_length(list)
 #define lives_list_free(list) g_list_free(list)
-#define lives_slist_free(list) g_slist_free(list)
 #define lives_list_append(list, data) g_list_append(list, data)
 #define lives_list_prepend(list, data) g_list_prepend(list, data)
 #define lives_list_find(list, data) g_list_find(list, data)
@@ -218,6 +207,7 @@ typedef gint(*LiVESCompareFunc)(gconstpointer a, gconstpointer b);
 #define lives_slist_length(list) g_slist_length(list)
 #define lives_slist_nth_data(list, i) g_slist_nth_data(list, i)
 #define lives_slist_append(list, data) g_slist_append(list, data)
+#define lives_slist_prepend(list, data) g_slist_prepend(list, data)
 
 #define lives_build_filename(...) g_build_filename(__VA_ARGS__)
 #define lives_build_path(...) g_build_path(LIVES_DIR_SEP, __VA_ARGS__)
@@ -257,6 +247,10 @@ typedef gint(*LiVESCompareFunc)(gconstpointer a, gconstpointer b);
 
 #define lives_path_get_dirname(a) g_path_get_dirname(a)
 
+#ifndef lives_locale_to_utf8
+#define lives_locale_to_utf8(a, b, c, d, e) g_locale_to_utf8(a, b, c, d, e)
+#endif
+
 #define U82L(String) (g_locale_from_utf8(String, -1, NULL, NULL, NULL))
 #define L2U8(String) (g_locale_to_utf8(String, -1, NULL, NULL, NULL))
 
@@ -332,6 +326,7 @@ typedef GdkDeviceManager                  LiVESXDeviceManager;
 #define EXPOSE_FN_END }
 
 #if GTK_CHECK_VERSION(3, 0, 0)
+#define NO_ALT_VALUES
 #if GTK_CHECK_VERSION(4, 0, 0)
 #define LIVES_WIDGET_EXPOSE_EVENT "snapshot"
 #else
@@ -347,10 +342,13 @@ typedef GdkDeviceManager                  LiVESXDeviceManager;
 #define lives_widget_object_set_data(a, b, c) g_object_set_data(a, b, c)
 #define lives_widget_object_get_data(a, b) g_object_get_data(a, b)
 
+#define lives_widget_object_set(a, b, c) g_object_set(a, b, c, NULL)
+#define lives_widget_object_get(a, b, c) g_object_get(a, b, c, NULL)
+
 #define LIVES_WIDGET_OBJECT(a) G_OBJECT(a)
 
-#define lives_object_freeze_notify(a) g_object_freeze_notify(a)
-#define lives_object_thaw_notify(a) g_object_thaw_notify(a)
+#define lives_widget_object_freeze_notify(a) g_object_freeze_notify(a)
+#define lives_widget_object_thaw_notify(a) g_object_thaw_notify(a)
 
 #if GTK_CHECK_VERSION(3, 0, 0)
 #define NO_MEM_OVERRIDE TRUE
@@ -368,7 +366,18 @@ typedef GdkFilterReturn LiVESFilterReturn;
 #define LIVES_FILTER_REMOVE GDK_FILTER_REMOVE
 #define LIVES_FILTER_CONTINUE GDK_FILTER_CONTINUE
 
+// font sizes
+#define LIVES_FONT_SIZE_XX_SMALL "xx-small" // 0
+#define LIVES_FONT_SIZE_X_SMALL "x-small"  // 1
+#define LIVES_FONT_SIZE_SMALL "small"      // 2
+#define LIVES_FONT_SIZE_MEDIUM "medium"   // 3
+#define LIVES_FONT_SIZE_LARGE "large"      // 4
+#define LIVES_FONT_SIZE_X_LARGE "x-large"    // 5
+#define LIVES_FONT_SIZE_XX_LARGE "xx-large"   // 6
+#define LIVES_FONT_SIZE_NORMAL LIVES_FONT_SIZE_MEDIUM
+
 // events
+#define LIVES_WIDGET_EVENT "event"
 #define LIVES_WIDGET_SCROLL_EVENT "scroll-event"
 #define LIVES_WIDGET_CONFIGURE_EVENT "configure-event"
 #define LIVES_WIDGET_ENTER_EVENT "enter-notify-event"
@@ -378,6 +387,7 @@ typedef GdkFilterReturn LiVESFilterReturn;
 #define LIVES_WIDGET_LEAVE_NOTIFY_EVENT "leave-notify-event"
 #define LIVES_WIDGET_FOCUS_OUT_EVENT "focus-out-event"
 #define LIVES_WIDGET_DELETE_EVENT "delete-event"
+#define LIVES_WIDGET_DESTROY_EVENT "destroy"
 #define LIVES_WIDGET_KEY_PRESS_EVENT "key-press-event"
 #define LIVES_WIDGET_KEY_RELEASE_EVENT "key-release-event"
 
@@ -393,13 +403,14 @@ typedef GdkFilterReturn LiVESFilterReturn;
 #define LIVES_WIDGET_DRAG_DATA_RECEIVED_SIGNAL "drag-data-received"
 #define LIVES_WIDGET_SIZE_PREPARED_SIGNAL "size-prepared"
 #define LIVES_WIDGET_MODE_CHANGED_SIGNAL "mode-changed"
-#define LIVES_WIDGET_ACCEPT_POSITION_SIGNAL "accept-position"
 #define LIVES_WIDGET_SWITCH_PAGE_SIGNAL "switch-page"
 #define LIVES_WIDGET_UNMAP_SIGNAL "unmap"
 #define LIVES_WIDGET_EDITED_SIGNAL "edited"
 #define LIVES_WIDGET_ROW_EXPANDED_SIGNAL "row-expanded"
 #define LIVES_WIDGET_COLOR_SET_SIGNAL "color-set"
 #define LIVES_WIDGET_SET_FOCUS_CHILD_SIGNAL "set-focus-child"
+#define LIVES_WIDGET_SHOW_SIGNAL "show"
+#define LIVES_WIDGET_HIDE_SIGNAL "hide"
 
 #if GTK_CHECK_VERSION(3, 0, 0)
 #define LIVES_WIDGET_STATE_CHANGED_SIGNAL "state-flags-changed"
@@ -407,6 +418,8 @@ typedef GdkFilterReturn LiVESFilterReturn;
 #define LIVES_WIDGET_STATE_CHANGED_SIGNAL "state-changed"
 #endif
 
+#define LIVES_WIDGET_NOTIFY_SIGNAL "notify::"
+
 typedef GtkWidget                         LiVESWidget;
 typedef GtkWindow                         LiVESWindow;
 typedef GtkContainer                      LiVESContainer;
@@ -499,7 +512,7 @@ typedef GtkGrid                           LiVESGrid;
 typedef LiVESWidget                       LiVESGrid;
 #endif
 
-#ifdef LIVES_TABLE_IS_GRID
+#if LIVES_TABLE_IS_GRID
 typedef GtkGrid                           LiVESTable;
 #else
 typedef GtkTable                          LiVESTable;
@@ -509,24 +522,27 @@ typedef GtkEditable                       LiVESEditable;
 
 #if GTK_CHECK_VERSION(3, 0, 0)
 #define LIVES_WIDGET_COLOR_HAS_ALPHA (1)
-#define LIVES_WIDGET_COLOR_SCALE(x) (x) ///< macro to get 0. to 1.
-#define LIVES_WIDGET_COLOR_STRETCH(x) (x*65535.) ///< macro to get 0. to 1.
+#define LIVES_WIDGET_COLOR_SCALE(x) (x) ///< macro to get 0. to 1. from widget colour
+#define LIVES_WIDGET_COLOR_STRETCH(x) (x*65535.) ///< macro to get 0. to 65535. from widget colour
 #define LIVES_WIDGET_COLOR_SCALE_65535(x) ((double)x/65535.) ///< macro to convert from (0. - 65535.) to widget color
 #define LIVES_WIDGET_COLOR_SCALE_255(x) ((double)x/255.) ///< macro to convert from (0. - 255.) to widget color
 typedef GdkRGBA                           LiVESWidgetColor; ///< component values are 0. to 1.
 
 typedef GtkStateFlags LiVESWidgetState;
 
-#define LIVES_WIDGET_STATE_NORMAL         GTK_STATE_FLAG_NORMAL
-#define LIVES_WIDGET_STATE_ACTIVE         GTK_STATE_FLAG_ACTIVE
-#define LIVES_WIDGET_STATE_PRELIGHT       GTK_STATE_FLAG_PRELIGHT
-#define LIVES_WIDGET_STATE_SELECTED       GTK_STATE_FLAG_SELECTED
-#define LIVES_WIDGET_STATE_INSENSITIVE    GTK_STATE_FLAG_INSENSITIVE
-#define LIVES_WIDGET_STATE_INCONSISTENT   GTK_STATE_FLAG_INCONSISTENT
-#define LIVES_WIDGET_STATE_FOCUSED        GTK_STATE_FLAG_FOCUSED
-#define LIVES_WIDGET_STATE_BACKDROP       GTK_STATE_FLAG_BACKDROP
-
+#define LIVES_WIDGET_STATE_NORMAL         GTK_STATE_FLAG_NORMAL        // 0
+#define LIVES_WIDGET_STATE_ACTIVE         GTK_STATE_FLAG_ACTIVE       //  1
+#define LIVES_WIDGET_STATE_PRELIGHT       GTK_STATE_FLAG_PRELIGHT     //  2
+#define LIVES_WIDGET_STATE_SELECTED       GTK_STATE_FLAG_SELECTED     //  4
+#define LIVES_WIDGET_STATE_INSENSITIVE    GTK_STATE_FLAG_INSENSITIVE     // 8
+#define LIVES_WIDGET_STATE_INCONSISTENT   GTK_STATE_FLAG_INCONSISTENT   // 16
+#define LIVES_WIDGET_STATE_FOCUSED        GTK_STATE_FLAG_FOCUSED         // 32
+#define LIVES_WIDGET_STATE_BACKDROP       GTK_STATE_FLAG_BACKDROP       // 64
+#if GTK_CHECK_VERSION(3, 14, 0)
+#define LIVES_WIDGET_STATE_CHECKED        GTK_STATE_FLAG_CHECKED
+#endif
 #else
+
 #define LIVES_WIDGET_COLOR_HAS_ALPHA (0)
 #define LIVES_WIDGET_COLOR_SCALE(x) ((double)x/65535.)     ///< macro to get 0. to 1. from widget color
 #define LIVES_WIDGET_COLOR_STRETCH(x) (x)     ///< macro to get 0 to 65535 from widget color
@@ -540,6 +556,7 @@ typedef GtkStateType LiVESWidgetState;
 #define LIVES_WIDGET_STATE_PRELIGHT       GTK_STATE_PRELIGHT
 #define LIVES_WIDGET_STATE_SELECTED       GTK_STATE_SELECTED
 #define LIVES_WIDGET_STATE_INSENSITIVE    GTK_STATE_INSENSITIVE
+
 #define LIVES_WIDGET_STATE_INCONSISTENT   (GTK_STATE_INSENSITIVE+1)
 #define LIVES_WIDGET_STATE_FOCUSED        (GTK_STATE_INSENSITIVE+2)
 #define LIVES_WIDGET_STATE_BACKDROP       (GTK_STATE_INSENSITIVE+3)
@@ -547,18 +564,19 @@ typedef GtkStateType LiVESWidgetState;
 
 typedef int LiVESResponseType;
 #define LIVES_RESPONSE_NONE GTK_RESPONSE_NONE
-#define LIVES_RESPONSE_OK GTK_RESPONSE_OK
-#define LIVES_RESPONSE_CANCEL GTK_RESPONSE_CANCEL
-#define LIVES_RESPONSE_ACCEPT GTK_RESPONSE_ACCEPT
-#define LIVES_RESPONSE_YES GTK_RESPONSE_YES
-#define LIVES_RESPONSE_NO GTK_RESPONSE_NO
+#define LIVES_RESPONSE_OK GTK_RESPONSE_OK     // -6
+#define LIVES_RESPONSE_CANCEL GTK_RESPONSE_CANCEL  // -5
+#define LIVES_RESPONSE_ACCEPT GTK_RESPONSE_ACCEPT  // 
+#define LIVES_RESPONSE_YES GTK_RESPONSE_YES  // -8
+#define LIVES_RESPONSE_NO GTK_RESPONSE_NO  // -9
 
 // positive values for custom responses
 #define LIVES_RESPONSE_INVALID 0
 #define LIVES_RESPONSE_RETRY 1
 #define LIVES_RESPONSE_ABORT 2
-#define LIVES_RESPONSE_RESET 3
+#define LIVES_RESPONSE_RESET 3  // be careful NOT to confuse this with RETRY !!
 #define LIVES_RESPONSE_SHOW_DETAILS 4
+#define LIVES_RESPONSE_BROWSE 5
 
 typedef GtkAttachOptions LiVESAttachOptions;
 #define LIVES_EXPAND GTK_EXPAND
@@ -590,12 +608,13 @@ typedef GtkFileChooserAction LiVESFileChooserAction;
 
 typedef GtkIconSize LiVESIconSize;
 #define LIVES_ICON_SIZE_INVALID GTK_ICON_SIZE_INVALID
-#define LIVES_ICON_SIZE_MENU GTK_ICON_SIZE_MENU
-#define LIVES_ICON_SIZE_SMALL_TOOLBAR GTK_ICON_SIZE_SMALL_TOOLBAR
-#define LIVES_ICON_SIZE_LARGE_TOOLBAR GTK_ICON_SIZE_LARGE_TOOLBAR
-#define LIVES_ICON_SIZE_BUTTON GTK_ICON_SIZE_BUTTON
-#define LIVES_ICON_SIZE_DND GTK_ICON_SIZE_DND
-#define LIVES_ICON_SIZE_DIALOG GTK_ICON_SIZE_DIALOG
+#define LIVES_ICON_SIZE_MENU GTK_ICON_SIZE_MENU // 16px
+#define LIVES_ICON_SIZE_SMALL_TOOLBAR GTK_ICON_SIZE_SMALL_TOOLBAR // 16px
+#define LIVES_ICON_SIZE_LARGE_TOOLBAR GTK_ICON_SIZE_LARGE_TOOLBAR // 24px
+#define LIVES_ICON_SIZE_BUTTON GTK_ICON_SIZE_BUTTON // 16px
+#define LIVES_ICON_SIZE_DND GTK_ICON_SIZE_DND // 32px
+#define LIVES_ICON_SIZE_DIALOG GTK_ICON_SIZE_DIALOG // 48px
+#define LIVES_ICON_SIZE_CUSTOM 1024
 
 // scrolledwindow policies
 typedef GtkPolicyType LiVESPolicyType;
@@ -616,6 +635,25 @@ typedef GtkArrowType LiVESArrowType;
 #define LIVES_ARROW_RIGHT GTK_ARROW_RIGHT
 #define LIVES_ARROW_NONE GTK_ARROW_NONE
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+typedef GtkAlign LiVESAlign;
+#define LIVES_ALIGN_FILL GTK_ALIGN_FILL
+#define LIVES_ALIGN_START GTK_ALIGN_START
+#define LIVES_ALIGN_END GTK_ALIGN_END
+#define LIVES_ALIGN_CENTER GTK_ALIGN_CENTER
+#else
+typedef int LiVESAlign;
+#define LIVES_ALIGN_FILL 0
+#define LIVES_ALIGN_START 1
+#define LIVES_ALIGN_END 2
+#define LIVES_ALIGN_CENTER 3
+#endif
+
+
+#if GTK_CHECK_VERSION(3, 10, 0)
+#define LIVES_ALIGN_BASELINE GTK_ALIGN_BASELINE
+#endif
+
 typedef GtkWrapMode LiVESWrapMode;
 #define LIVES_WRAP_NONE GTK_WRAP_NONE
 //#define LIVES_WRAP_CHAR GTK_WRAP_CHAR
@@ -819,8 +857,10 @@ typedef GdkInterpType                     LiVESInterpType;
 
 #if LIVES_TABLE_IS_GRID
 #define LIVES_TABLE(widget) GTK_GRID(widget)
+#define LIVES_IS_TABLE(widget) GTK_IS_GRID(widget)
 #else
 #define LIVES_TABLE(widget) GTK_TABLE(widget)
+#define LIVES_IS_TABLE(widget) GTK_IS_TABLE(widget)
 #endif
 
 #define LIVES_RANGE(widget) GTK_RANGE(widget)
@@ -830,16 +870,19 @@ typedef GdkInterpType                     LiVESInterpType;
 #define LIVES_XEVENT(event) GDK_EVENT(event)
 
 #define LIVES_IS_WIDGET_OBJECT(object) G_IS_OBJECT(object)
+#define LIVES_IS_OBJECT(object) G_IS_OBJECT(object)
 #define LIVES_IS_WIDGET(widget) GTK_IS_WIDGET(widget)
 #define LIVES_IS_WINDOW(widget) GTK_IS_WINDOW(widget)
 #define LIVES_IS_XWINDOW(widget) GDK_IS_WINDOW(widget)
 #define LIVES_IS_PIXBUF(widget) GDK_IS_PIXBUF(widget)
 #define LIVES_IS_CONTAINER(widget) GTK_IS_CONTAINER(widget)
 #define LIVES_IS_BIN(widget) GTK_IS_BIN(widget)
+#define LIVES_IS_SCROLLBAR(widget) GTK_IS_SCROLLBAR(widget)
+#define LIVES_IS_TOOL_BUTTON(widget) GTK_IS_TOOL_BUTTON(widget)
 
 #if GTK_CHECK_VERSION(3, 0, 0)
 #define LIVES_IS_HBOX(widget) (GTK_IS_BOX(widget) && gtk_orientable_get_orientation(GTK_ORIENTABLE(widget)) == GTK_ORIENTATION_HORIZONTAL)
-#define LIVES_IS_VBOX(widget) (GTK_IS_BOX(widget) && gtk_orientable_get_orientation(GTK_ORIENTABLE(widget)) == GTK_ORIENTATION_HORIZONTAL)
+#define LIVES_IS_VBOX(widget) (GTK_IS_BOX(widget) && gtk_orientable_get_orientation(GTK_ORIENTABLE(widget)) == GTK_ORIENTATION_VERTICAL)
 #define LIVES_IS_SCROLLABLE(widget) GTK_IS_SCROLLABLE(widget)
 #else
 #define LIVES_IS_HBOX(widget) GTK_IS_HBOX(widget)
@@ -877,6 +920,7 @@ typedef GdkInterpType                     LiVESInterpType;
 #define LIVES_STOCK_APPLY "gtk-apply"      // non-standard image ?
 #define LIVES_STOCK_CANCEL "gtk-cancel"    // non-standard image ?
 #define LIVES_STOCK_OK "gtk-ok"    // non-standard image ?
+
 #define LIVES_STOCK_UNDO "edit-undo"
 #define LIVES_STOCK_REDO "edit-redo"
 #define LIVES_STOCK_ADD "list-add"
@@ -1014,6 +1058,19 @@ char LIVES_STOCK_LABEL_MEDIA_RECORD[32];
 
 #endif
 
+//#define LIVES_LIVES_STOCK_LOCKED "lives-locked"
+//#define LIVES_LIVES_STOCK_UNLOCKED "lives-unlocked"
+
+#if GTK_CHECK_VERSION(3, 2, 0)
+#define LIVES_LIVES_STOCK_LOCKED "changes-prevent"
+#define LIVES_LIVES_STOCK_UNLOCKED "changes-allow"
+#else
+#define LIVES_LIVES_STOCK_LOCKED "locked"
+#define LIVES_LIVES_STOCK_UNLOCKED "unlocked"
+#endif
+
+#define LIVES_LIVES_STOCK_LOOP "lives-loop"
+
 #define LIVES_DEFAULT_MOD_MASK (gtk_accelerator_get_default_mod_mask ())
 
 #define LIVES_CONTROL_MASK GDK_CONTROL_MASK
@@ -1179,21 +1236,6 @@ char LIVES_STOCK_LABEL_MEDIA_RECORD[32];
 #define LIVES_UINT_TO_POINTER GUINT_TO_POINTER
 #define LIVES_POINTER_TO_INT  GPOINTER_TO_INT
 
-// pango stuff
-typedef PangoLayout LingoLayout;
-#define lingo_layout_set_alignment(a, b) pango_layout_set_alignment(a, b)
-
-#define LINGO_ALIGN_LEFT PANGO_ALIGN_LEFT
-#define LINGO_ALIGN_RIGHT PANGO_ALIGN_RIGHT
-#define LINGO_ALIGN_CENTER PANGO_ALIGN_CENTER
-
-#define lingo_layout_set_text(a, b, c) pango_layout_set_text(a, b, c)
-#define lingo_painter_show_layout(a, b) pango_cairo_show_layout(a, b)
-
-#define lingo_layout_get_size(a, b, c, d, e) pango_layout_get_size(a, b, c)
-
-#define LINGO_SCALE PANGO_SCALE
-
 #endif
 
 #endif
