@@ -23,6 +23,12 @@
 #define LIVES_HAS_IMAGE_MENU_ITEM 0
 #define LIVES_HAS_DEVICE_MANAGER 0
 
+#define MIN_SPINBUTTON_SIZE 6 ///< min digits for spinbuttons
+
+#define LONG_ENTRY_WIDTH ((int)(120.*widget_opts.scale))
+#define SHORT_ENTRY_WIDTH ((int)(40.*widget_opts.scale))
+#define MEDIUM_ENTRY_WIDTH ((int)(60.*widget_opts.scale))
+
 typedef enum {
   LIVES_DISPLAY_TYPE_UNKNOWN = 0,
   LIVES_DISPLAY_TYPE_X11,
@@ -32,29 +38,97 @@ typedef enum {
 
 // values below are multiplied by scale
 #define W_PACKING_WIDTH  10 // packing width for widgets with labels
-#define W_PACKING_HEIGHT 10 // packing height for widgets
+#define W_PACKING_HEIGHT 6 // packing height for widgets
 #define W_BORDER_WIDTH   10 // default border width
-#define W_FILL_LENGTH    200 // default extra fill size
+#define W_FILL_LENGTH    (W_PACKING_WIDTH * 4) // default extra fill size
 
 #define ulong_random() lives_random()
 
-#if defined (GUI_GTK) || defined (PAINTER_CAIRO)
+#if defined (GUI_GTK) || defined (LIVES_PAINTER_IS_CAIRO)
 #include "widget-helper-gtk.h"
 #endif
 
 // basic functions (wrappers for Toolkit functions)
 
+#ifdef LIVES_LINGO_IS_PANGO
+// pango stuff. I suppose it should be here on the offchance that it might one day be used with a non-gtk+ toolkit
+typedef PangoLayout LingoLayout;
+typedef PangoContext LingoContext;
+#define lingo_layout_set_alignment(a, b) pango_layout_set_alignment(a, b)
+
+#define LINGO_ALIGN_LEFT PANGO_ALIGN_LEFT
+#define LINGO_ALIGN_RIGHT PANGO_ALIGN_RIGHT
+#define LINGO_ALIGN_CENTER PANGO_ALIGN_CENTER
+
+#define lingo_layout_set_text(a, b, c) pango_layout_set_text(a, b, c)
+#ifdef LIVES_PAINTER_IS_CAIRO
+#define lingo_painter_show_layout(a, b) pango_cairo_show_layout(a, b)
+#endif
+#ifdef GUI_GTK
+#define lives_widget_create_lingo_context(a) gtk_widget_create_pango_context(a)
+#endif
+#define lingo_layout_get_size(a, b, c, d, e) pango_layout_get_size(a, b, c)
+#define lingo_layout_new(a) pango_layout_new(a)
+#define lingo_layout_set_markup(a, b, c) pango_layout_set_markup(a, b, c)
+
+#define LINGO_IS_LAYOUT(a) PANGO_IS_LAYOUT(a)
+#define LINGO_IS_CONTEXT(a) PANGO_IS_CONTEXT(a)
+
+#define LINGO_SCALE PANGO_SCALE
+#endif
+
+#ifdef LIVES_PAINTER_IS_CAIRO
+// likewise with cairo
+#ifndef GUI_GTK
+#include <cairo/cairo.h>
+#endif
+
+typedef cairo_t lives_painter_t;
+typedef cairo_surface_t lives_painter_surface_t;
+
+boolean lives_painter_surface_destroy(lives_painter_surface_t *);
+
+typedef cairo_format_t lives_painter_format_t;
+
+#define LIVES_PAINTER_FORMAT_A1   CAIRO_FORMAT_A1
+#define LIVES_PAINTER_FORMAT_A8   CAIRO_FORMAT_A8
+#define LIVES_PAINTER_FORMAT_RGB24 CAIRO_FORMAT_RGB24
+#define LIVES_PAINTER_FORMAT_ARGB32 CAIRO_FORMAT_ARGB32
+
+typedef cairo_content_t lives_painter_content_t; // eg. color, alpha, color+alpha
+
+#define LIVES_PAINTER_CONTENT_COLOR CAIRO_CONTENT_COLOR
+
+typedef cairo_operator_t lives_painter_operator_t;
+
+#define LIVES_PAINTER_OPERATOR_UNKNOWN CAIRO_OPERATOR_OVER
+#define LIVES_PAINTER_OPERATOR_DEFAULT CAIRO_OPERATOR_OVER
+
+#define LIVES_PAINTER_OPERATOR_DEST_OUT CAIRO_OPERATOR_DEST_OUT
+#if CAIRO_VERSION < CAIRO_VERSION_ENCODE(1, 10, 0)
+#define LIVES_PAINTER_OPERATOR_DIFFERENCE CAIRO_OPERATOR_OVER
+#define LIVES_PAINTER_OPERATOR_OVERLAY CAIRO_OPERATOR_OVER
+#else
+#define LIVES_PAINTER_OPERATOR_DIFFERENCE CAIRO_OPERATOR_DIFFERENCE
+#define LIVES_PAINTER_OPERATOR_OVERLAY CAIRO_OPERATOR_OVERLAY
+#endif
+
+typedef cairo_fill_rule_t lives_painter_fill_rule_t;
+
+#define LIVES_PAINTER_FILL_RULE_WINDING  CAIRO_FILL_RULE_WINDING
+#define LIVES_PAINTER_FILL_RULE_EVEN_ODD CAIRO_FILL_RULE_EVEN_ODD
+
+#endif
+
 // lives_painter_functions
 
-lives_painter_t *lives_painter_create(lives_painter_surface_t *target);
+lives_painter_t *lives_painter_create_from_surface(lives_painter_surface_t *target);
 lives_painter_t *lives_painter_create_from_widget(LiVESWidget *);
 boolean lives_painter_set_source_pixbuf(lives_painter_t *, const LiVESPixbuf *, double pixbuf_x, double pixbuf_y);
 boolean lives_painter_set_source_surface(lives_painter_t *, lives_painter_surface_t *, double x, double y);
 lives_painter_surface_t *lives_painter_image_surface_create(lives_painter_format_t format, int width, int height);
 lives_painter_surface_t *lives_painter_image_surface_create_for_data(uint8_t *data, lives_painter_format_t,
     int width, int height, int stride);
-lives_painter_surface_t *lives_painter_surface_create_from_widget(LiVESWidget *, lives_painter_content_t,
-    int width, int height);
 boolean lives_painter_surface_flush(lives_painter_surface_t *);
 
 boolean lives_painter_destroy(lives_painter_t *);
@@ -96,7 +170,9 @@ lives_painter_format_t lives_painter_image_surface_get_format(lives_painter_surf
 // utils
 
 boolean widget_helper_init(void);
+boolean widget_opts_rescale(double scale);
 
+lives_colRGBA64_t lives_rgba_col_new(int red, int green, int blue, int alpha);
 boolean widget_color_to_lives_rgba(lives_colRGBA64_t *, LiVESWidgetColor *);
 boolean lives_rgba_to_widget_color(LiVESWidgetColor *, lives_colRGBA64_t *);
 
@@ -109,19 +185,11 @@ boolean lives_rgba_copy(lives_colRGBA64_t *col1, lives_colRGBA64_t *col2);
 
 // object funcs.
 
-livespointer lives_object_ref(livespointer); ///< increase refcount by one
+boolean lives_object_ref(livespointer); ///< increase refcount by one
 boolean lives_object_unref(livespointer); ///< decrease refcount by one: if refcount==0, object is destroyed
 
 // remove any "floating" reference and add a new ref
-#ifdef GUI_GTK
-#if GTK_CHECK_VERSION(3, 0, 0)
-livespointer lives_object_ref_sink(livespointer);
-#else
-void lives_object_ref_sink(livespointer);
-#endif
-#else
-livespointer lives_object_ref_sink(livespointer);
-#endif
+boolean lives_object_ref_sink(livespointer);
 
 // lives_pixbuf functions
 
@@ -154,6 +222,7 @@ boolean lives_pixbuf_saturate_and_pixelate(const LiVESPixbuf *src, LiVESPixbuf *
 #ifdef GUI_GTK
 
 #define lives_signal_connect(instance, detailed_signal, c_handler, data) g_signal_connect(instance, detailed_signal, c_handler, data)
+#define lives_signal_connect_swapped(instance, detailed_signal, c_handler, data) g_signal_connect_swapped(instance, detailed_signal, c_handler, data)
 #define lives_signal_connect_after(instance, detailed_signal, c_handler, data) g_signal_connect_after(instance, detailed_signal, c_handler, data)
 #define lives_signal_handlers_disconnect_by_func(instance, func, data) g_signal_handlers_disconnect_by_func(instance, func, data)
 #define lives_signal_handlers_block_by_func(instance, func, data) g_signal_handlers_block_by_func(instance, func, data)
@@ -177,9 +246,11 @@ boolean lives_widget_set_sensitive(LiVESWidget *, boolean state);
 boolean lives_widget_get_sensitive(LiVESWidget *);
 
 boolean lives_widget_show(LiVESWidget *);
+boolean lives_widget_show_now(LiVESWidget *);
 boolean lives_widget_show_all(LiVESWidget *);
 boolean lives_widget_hide(LiVESWidget *);
 boolean lives_widget_destroy(LiVESWidget *);
+boolean lives_widget_realize(LiVESWidget *);
 
 boolean lives_widget_queue_draw(LiVESWidget *);
 boolean lives_widget_queue_draw_area(LiVESWidget *, int x, int y, int width, int height);
@@ -194,6 +265,10 @@ boolean lives_widget_is_ancestor(LiVESWidget *, LiVESWidget *ancestor);
 boolean lives_widget_set_app_paintable(LiVESWidget *, boolean paintable);
 
 boolean lives_widget_has_focus(LiVESWidget *);
+boolean lives_widget_has_default(LiVESWidget *);
+
+boolean lives_widget_set_halign(LiVESWidget *, LiVESAlign align);
+boolean lives_widget_set_valign(LiVESWidget *, LiVESAlign align);
 
 LiVESWidget *lives_event_box_new(void);
 boolean lives_event_box_set_above_child(LiVESEventBox *, boolean set);
@@ -203,7 +278,7 @@ LiVESWidget *lives_label_new(const char *text);
 const char *lives_label_get_text(LiVESLabel *);
 boolean lives_label_set_text(LiVESLabel *, const char *text);
 
-boolean lives_label_set_xalign(LiVESLabel *, double align);
+//boolean lives_label_set_xalign(LiVESLabel *, double align);
 
 boolean lives_label_set_markup(LiVESLabel *, const char *markup);
 
@@ -212,15 +287,21 @@ LiVESWidget *lives_label_get_mnemonic_widget(LiVESLabel *);
 
 boolean lives_label_set_selectable(LiVESLabel *, boolean setting);
 
+//////////
+
 LiVESWidget *lives_button_new(void);
 LiVESWidget *lives_button_new_from_stock(const char *stock_id, const char *label);
 LiVESWidget *lives_button_new_with_label(const char *label);
 
 boolean lives_button_set_label(LiVESButton *, const char *label);
+LiVESWidget *lives_button_get_label_widget(LiVESButton *);
 
 boolean lives_button_set_relief(LiVESButton *, LiVESReliefStyle);
 boolean lives_button_set_image(LiVESButton *, LiVESWidget *image);
 boolean lives_button_set_focus_on_click(LiVESButton *, boolean focus);
+boolean lives_widget_set_focus_on_click(LiVESWidget *, boolean focus);
+
+/////////////////////////////
 
 LiVESWidget *lives_check_button_new(void);
 LiVESWidget *lives_check_button_new_with_label(const char *label);
@@ -237,6 +318,11 @@ boolean lives_widget_set_fg_color(LiVESWidget *, LiVESWidgetState state, const L
 boolean lives_widget_set_text_color(LiVESWidget *, LiVESWidgetState state, const LiVESWidgetColor *);
 boolean lives_widget_set_base_color(LiVESWidget *, LiVESWidgetState state, const LiVESWidgetColor *);
 
+boolean lives_widget_set_border_color(LiVESWidget *, LiVESWidgetState state, const LiVESWidgetColor *);
+boolean lives_widget_set_outline_color(LiVESWidget *, LiVESWidgetState state, const LiVESWidgetColor *);
+
+boolean lives_widget_set_font_size(LiVESWidget *, LiVESWidgetState state, const char *size);
+
 boolean lives_widget_get_fg_state_color(LiVESWidget *, LiVESWidgetState state, LiVESWidgetColor *);
 boolean lives_widget_get_bg_state_color(LiVESWidget *, LiVESWidgetState state, LiVESWidgetColor *);
 
@@ -244,6 +330,7 @@ boolean lives_color_parse(const char *spec, LiVESWidgetColor *);
 
 LiVESWidgetColor *lives_widget_color_copy(LiVESWidgetColor *c1orNULL, const LiVESWidgetColor *c2);
 boolean lives_widget_color_equal(LiVESWidgetColor *, const LiVESWidgetColor *);
+boolean lives_widget_color_mix(LiVESWidgetColor *c1, const LiVESWidgetColor *c2, float mixval);
 
 LiVESWidget *lives_image_new(void);
 LiVESWidget *lives_image_new_from_file(const char *filename);
@@ -252,6 +339,11 @@ LiVESWidget *lives_image_new_from_stock(const char *stock_id, LiVESIconSize size
 boolean lives_image_set_from_pixbuf(LiVESImage *, LiVESPixbuf *);
 LiVESPixbuf *lives_image_get_pixbuf(LiVESImage *);
 
+boolean lives_widget_set_margin_left(LiVESWidget *, int margin);
+boolean lives_widget_set_margin_right(LiVESWidget *, int margin);
+boolean lives_widget_set_margin_top(LiVESWidget *, int margin);
+boolean lives_widget_set_margin_bottom(LiVESWidget *, int margin);
+
 LiVESWidget *lives_dialog_get_content_area(LiVESDialog *);
 LiVESWidget *lives_dialog_get_action_area(LiVESDialog *);
 
@@ -314,13 +406,16 @@ LiVESWidget *lives_vbutton_box_new(void);
 boolean lives_button_box_set_layout(LiVESButtonBox *, LiVESButtonBoxStyle bstyle);
 boolean lives_button_box_set_button_width(LiVESButtonBox *, LiVESWidget *button, int min_width);
 
-LiVESWidget *lives_hscale_new(LiVESAdjustment *);
+boolean lives_button_set_border_colour(LiVESWidget *, LiVESWidgetState state, LiVESWidgetColor *);
+
+LiVESWidget *lives_standard_hscale_new(LiVESAdjustment *);
 LiVESWidget *lives_vscale_new(LiVESAdjustment *);
 
 LiVESWidget *lives_hpaned_new(void);
 LiVESWidget *lives_vpaned_new(void);
 
 boolean lives_paned_set_position(LiVESPaned *, int pos);
+int lives_paned_get_position(LiVESPaned *);
 boolean lives_paned_pack(int where, LiVESPaned *, LiVESWidget *child, boolean resize, boolean shrink);
 
 LiVESWidget *lives_hscrollbar_new(LiVESAdjustment *);
@@ -333,13 +428,15 @@ boolean lives_alignment_set(LiVESWidget *, float xalign, float yalign, float xsc
 
 LiVESWidget *lives_expander_new(const char *label);
 LiVESWidget *lives_expander_get_label_widget(LiVESExpander *expander);
+boolean lives_expander_set_use_markup(LiVESExpander *expander, boolean val);
 
 boolean lives_label_set_width_chars(LiVESLabel *label, int nchars);
 boolean lives_label_set_halignment(LiVESLabel *, float yalign);
 
 LiVESWidget *lives_combo_new(void);
-LiVESWidget *lives_combo_new_with_model(LiVESTreeModel *);
+LiVESWidget *lives_combo_new_with_model(LiVESTreeModel *model);
 LiVESTreeModel *lives_combo_get_model(LiVESCombo *);
+boolean lives_combo_set_model(LiVESCombo *, LiVESTreeModel *);
 
 boolean lives_combo_append_text(LiVESCombo *, const char *text);
 boolean lives_combo_set_entry_text_column(LiVESCombo *, int column);
@@ -365,8 +462,6 @@ boolean lives_text_view_set_cursor_visible(LiVESTextView *, boolean setting);
 boolean lives_text_view_set_wrap_mode(LiVESTextView *, LiVESWrapMode wrapmode);
 boolean lives_text_view_set_justification(LiVESTextView *, LiVESJustification justify);
 
-boolean lives_text_view_scroll_mark_onscreen(LiVESTextView *, LiVESTextMark *mark);
-
 LiVESTextBuffer *lives_text_buffer_new(void);
 char *lives_text_buffer_get_text(LiVESTextBuffer *tbuff, LiVESTextIter *start, LiVESTextIter *end, boolean inc_hidden_chars);
 boolean lives_text_buffer_set_text(LiVESTextBuffer *, const char *, int len);
@@ -402,6 +497,7 @@ int *lives_tree_path_get_indices(LiVESTreePath *);
 
 LiVESTreeStore *lives_tree_store_new(int ncols, ...);
 boolean lives_tree_store_append(LiVESTreeStore *, LiVESTreeIter *, LiVESTreeIter *parent);
+boolean lives_tree_store_prepend(LiVESTreeStore *, LiVESTreeIter *, LiVESTreeIter *parent);
 boolean lives_tree_store_set(LiVESTreeStore *, LiVESTreeIter *, ...);
 
 LiVESWidget *lives_tree_view_new(void);
@@ -437,6 +533,7 @@ int lives_event_get_time(LiVESXEvent *);
 boolean lives_toggle_button_get_active(LiVESToggleButton *);
 boolean lives_toggle_button_set_active(LiVESToggleButton *, boolean active);
 boolean lives_toggle_button_set_mode(LiVESToggleButton *, boolean drawind);
+boolean lives_toggle_button_toggle(LiVESToggleButton *);
 
 LiVESWidget *lives_toggle_tool_button_new(void);
 boolean lives_toggle_tool_button_get_active(LiVESToggleToolButton *);
@@ -462,6 +559,8 @@ boolean lives_widget_set_events(LiVESWidget *, int events);
 boolean lives_widget_remove_accelerator(LiVESWidget *, LiVESAccelGroup *, uint32_t accel_key, LiVESXModifierType accel_mods);
 boolean lives_widget_get_preferred_size(LiVESWidget *, LiVESRequisition *min_size, LiVESRequisition *nat_size);
 
+boolean lives_widget_set_no_show_all(LiVESWidget *, boolean set);
+
 boolean lives_container_remove(LiVESContainer *, LiVESWidget *);
 boolean lives_container_add(LiVESContainer *, LiVESWidget *);
 boolean lives_container_set_border_width(LiVESContainer *, uint32_t width);
@@ -479,12 +578,17 @@ double lives_spin_button_get_value(LiVESSpinButton *);
 int lives_spin_button_get_value_as_int(LiVESSpinButton *);
 
 LiVESAdjustment *lives_spin_button_get_adjustment(LiVESSpinButton *);
+LiVESAdjustment *lives_spin_button_set_adjustment(LiVESSpinButton *, LiVESAdjustment *adj);
 
 boolean lives_spin_button_set_value(LiVESSpinButton *, double value);
 boolean lives_spin_button_set_range(LiVESSpinButton *, double min, double max);
 
 boolean lives_spin_button_set_wrap(LiVESSpinButton *, boolean wrap);
 
+boolean lives_spin_button_set_step_increment(LiVESSpinButton *button, double step_increment);
+boolean lives_spin_button_set_snap_to_ticks(LiVESSpinButton *, boolean snap);
+boolean lives_spin_button_set_snap_to_multiples(LiVESSpinButton *, double mult);
+
 boolean lives_spin_button_set_digits(LiVESSpinButton *, uint32_t digits);
 
 boolean lives_spin_button_update(LiVESSpinButton *);
@@ -492,6 +596,8 @@ boolean lives_spin_button_update(LiVESSpinButton *);
 LiVESWidget *lives_color_button_new_with_color(const LiVESWidgetColor *);
 boolean lives_color_button_get_color(LiVESColorButton *, LiVESWidgetColor *);
 boolean lives_color_button_set_color(LiVESColorButton *, const LiVESWidgetColor *);
+boolean lives_color_button_set_alpha(LiVESColorButton *, int16_t alpha);
+int16_t lives_color_button_get_alpha(LiVESColorButton *);
 boolean lives_color_button_set_title(LiVESColorButton *, const char *title);
 boolean lives_color_button_set_use_alpha(LiVESColorButton *, boolean use_alpha);
 
@@ -508,7 +614,7 @@ LiVESWidget *lives_message_dialog_new(LiVESWindow *parent, LiVESDialogFlags flag
 double lives_ruler_get_value(LiVESRuler *);
 double lives_ruler_set_value(LiVESRuler *, double value);
 
-void lives_ruler_set_range(LiVESRuler *, double lower, double upper, double position, double max_size);
+boolean lives_ruler_set_range(LiVESRuler *, double lower, double upper, double position, double max_size);
 double lives_ruler_set_upper(LiVESRuler *, double upper);
 double lives_ruler_set_lower(LiVESRuler *, double lower);
 
@@ -536,11 +642,13 @@ boolean lives_widget_is_realized(LiVESWidget *);
 double lives_adjustment_get_upper(LiVESAdjustment *);
 double lives_adjustment_get_lower(LiVESAdjustment *);
 double lives_adjustment_get_page_size(LiVESAdjustment *);
+double lives_adjustment_get_step_increment(LiVESAdjustment *);
 double lives_adjustment_get_value(LiVESAdjustment *);
 
 boolean lives_adjustment_set_upper(LiVESAdjustment *, double upper);
 boolean lives_adjustment_set_lower(LiVESAdjustment *, double lower);
 boolean lives_adjustment_set_page_size(LiVESAdjustment *, double page_size);
+boolean lives_adjustment_set_step_increment(LiVESAdjustment *, double step_increment);
 boolean lives_adjustment_set_value(LiVESAdjustment *, double value);
 
 boolean lives_adjustment_clamp_page(LiVESAdjustment *, double lower, double upper);
@@ -554,10 +662,12 @@ boolean lives_range_set_inverted(LiVESRange *, boolean invert);
 double lives_range_get_value(LiVESRange *);
 
 boolean lives_editable_set_editable(LiVESEditable *, boolean editable);
+boolean lives_editable_get_editable(LiVESEditable *);
 boolean lives_editable_select_region(LiVESEditable *, int start_pos, int end_pos);
 
 LiVESWidget *lives_entry_new(void);
 boolean lives_entry_set_editable(LiVESEntry *, boolean editable);
+boolean lives_entry_get_editable(LiVESEntry *);
 const char *lives_entry_get_text(LiVESEntry *);
 boolean lives_entry_set_text(LiVESEntry *, const char *text);
 boolean lives_entry_set_width_chars(LiVESEntry *, int nchars);
@@ -639,6 +749,7 @@ boolean lives_image_menu_item_set_image(LiVESImageMenuItem *, LiVESWidget *image
 #endif
 
 boolean lives_menu_item_set_submenu(LiVESMenuItem *, LiVESWidget *);
+LiVESWidget *lives_menu_item_get_submenu(LiVESMenuItem *);
 
 boolean lives_menu_item_activate(LiVESMenuItem *);
 
@@ -687,6 +798,9 @@ LiVESAdjustment *lives_scrolled_window_get_vadjustment(LiVESScrolledWindow *);
 boolean lives_scrolled_window_set_policy(LiVESScrolledWindow *, LiVESPolicyType hpolicy, LiVESPolicyType vpolicy);
 boolean lives_scrolled_window_add_with_viewport(LiVESScrolledWindow *, LiVESWidget *child);
 
+boolean lives_scrolled_window_set_min_content_height(LiVESScrolledWindow *, int height);
+boolean lives_scrolled_window_set_min_content_width(LiVESScrolledWindow *, int width);
+
 boolean lives_xwindow_raise(LiVESXWindow *);
 boolean lives_xwindow_set_cursor(LiVESXWindow *, LiVESXCursor *);
 
@@ -719,9 +833,19 @@ boolean lives_window_set_auto_startup_notification(boolean set);
 
 void lives_label_set_hpadding(LiVESLabel *label, int pad);
 
-boolean lives_widget_grab_default_special(LiVESWidget *);
+LiVESWidget *align_horizontal_with(LiVESWidget *thingtoadd, LiVESWidget *thingtoalignwith);
 
-#define BUTTON_DIM_VAL 24000 // fg / bg ratio for dimmed buttons (BUTTON_DIM_VAL/65535)
+// this is not the same as a GtkLayout
+LiVESWidget *lives_layout_new(LiVESBox *);
+LiVESWidget *lives_layout_hbox_new(LiVESTable *);
+int lives_layout_add_row(LiVESTable *layout);
+
+boolean lives_button_grab_default_special(LiVESWidget *);
+
+#define BUTTON_DIM_VAL (0.4 * 65535.) // fg / bg ratio for dimmed buttons (BUTTON_DIM_VAL/65535) (lower is dimmer)
+
+#define LOCK_BUTTON_WIDTH 24 ///< sizes for the lock button
+#define LOCK_BUTTON_HEIGHT 24
 
 LiVESWidget *lives_standard_button_new(void);
 LiVESWidget *lives_standard_button_new_with_label(const char *labeltext);
@@ -737,6 +861,8 @@ LiVESWidget *lives_standard_radio_menu_item_new_with_label(LiVESSList *group, co
 
 LiVESWidget *lives_standard_check_menu_item_new_with_label(const char *labeltext, boolean active);
 
+LiVESWidget *lives_standard_notebook_new(const LiVESWidgetColor *bg_color, const LiVESWidgetColor *act_color);
+
 LiVESWidget *lives_standard_label_new(const char *labeltext);
 LiVESWidget *lives_standard_label_new_with_mnemonic_widget(const char *text, LiVESWidget *mnemonic_widget);
 
@@ -745,19 +871,35 @@ LiVESWidget *lives_standard_drawing_area_new(LiVESGuiCallback callback, ulong *r
 LiVESWidget *lives_standard_frame_new(const char *labeltext, float xalign, boolean invisible_outline);
 
 LiVESWidget *lives_standard_check_button_new(const char *labeltext, boolean active, LiVESBox *, const char *tooltip);
+LiVESWidget *lives_glowing_check_button_new(const char *labeltext, LiVESBox *, const char *tooltip, boolean *togglevalue);
 LiVESWidget *lives_standard_radio_button_new(const char *labeltext, LiVESSList **rbgroup,
     LiVESBox *, const char *tooltip);
 LiVESWidget *lives_standard_spin_button_new(const char *labeltext, double val, double min,
     double max, double step, double page, int dp, LiVESBox *,
     const char *tooltip);
-LiVESWidget *lives_standard_combo_new(const char *labeltext, LiVESList *list, LiVESBox *,
-                                      const char *tooltip);
+LiVESWidget *lives_standard_combo_new(const char *labeltext, LiVESList *list, LiVESBox *, const char *tooltip);
+
+LiVESWidget *lives_standard_combo_new_with_model(LiVESTreeModel *, LiVESBox *);
 
 LiVESWidget *lives_standard_entry_new(const char *labeltext, const char *txt, int dispwidth, int maxchars, LiVESBox *,
                                       const char *tooltip);
 
+LiVESWidget *lives_standard_direntry_new(const char *labeltext, const char *txt, int dispwidth, int maxchars, LiVESBox *,
+    const char *tooltip);
+
+LiVESWidget *lives_standard_fileentry_new(const char *labeltext, const char *txt, const char *defdir, int dispwidth, int maxchars,
+    LiVESBox *box, const char *tooltip);
+
+LiVESToolItem *lives_menu_tool_button_new(LiVESWidget *icon, const char *label);
+
+LiVESWidget *lives_standard_lock_button_new(boolean is_locked, int width, int height, const char *tooltip);
+
+boolean lives_lock_button_get_locked(LiVESButton *lockbutton);
+
 LiVESWidget *lives_standard_dialog_new(const char *title, boolean add_std_buttons, int width, int height);
 
+LiVESWidget *lives_dialog_add_button_from_stock(LiVESDialog *dialog, const char *stock_id, const char *label, int response_id);
+
 LiVESWidget *lives_standard_hruler_new(void);
 
 LiVESWidget *lives_standard_scrolled_window_new(int width, int height, LiVESWidget *child);
@@ -773,29 +915,60 @@ LiVESWidget *lives_standard_color_button_new(LiVESBox *parent, const char *name,
 
 LiVESWidget *lives_standard_text_view_new(const char *text, LiVESTextBuffer *tbuff);
 
+LiVESToolItem *lives_standard_menu_tool_button_new(LiVESWidget *icon, const char *label);
+
 LiVESXCursor *lives_cursor_new_from_pixbuf(LiVESXDisplay *, LiVESPixbuf *, int x, int y);
 
+void set_button_image_border_colour(LiVESButton *, LiVESWidgetState state, LiVESWidgetColor *);
+
 // util functions
 
+// THEME COLOURS (will be done more logically in the future)
 void lives_widget_apply_theme(LiVESWidget *, LiVESWidgetState state); // normal theme colours
+void lives_widget_apply_theme_dimmed(LiVESWidget *widget, LiVESWidgetState state, int dimval); // dimmed normal theme
+void set_child_dimmed_colour(LiVESWidget *widget, int dim); // dimmed normal theme for children (insensitive state only)
+
+// if set_all, set the widget itself (labels always set_all; buttons are ignored if set_all is FALSE)
+void set_child_colour(LiVESWidget *, boolean set_all); // normal theme, sensitive and insensitive
+
 void lives_widget_apply_theme2(LiVESWidget *, LiVESWidgetState state, boolean set_fg); // menu and bars colours
+void lives_widget_apply_theme_dimmed2(LiVESWidget *widget, LiVESWidgetState state, int dimval);
+void set_child_dimmed_colour2(LiVESWidget *widget, int dim); // dimmed m & b for children (insensitive state only)
+
+// like set_child_colour, but with menu and bars colours
+void set_child_alt_colour(LiVESWidget *, boolean set_all);
+
 void lives_widget_apply_theme3(LiVESWidget *, LiVESWidgetState state); // info base/text
+void set_child_colour3(LiVESWidget *, boolean set_all);
+
+boolean lives_image_scale(LiVESImage *, int width, int height, LiVESInterpType interp_type);
+
+LiVESPixbuf *lives_pixbuf_new_from_stock_at_size(const char *stock_id, LiVESIconSize size, int x, int y);
+LiVESWidget *lives_image_new_from_stock_at_size(const char *stock_id, LiVESIconSize size, int x, int y);
+
+boolean lives_widget_queue_draw_if_visible(LiVESWidget *);
+boolean lives_widget_queue_draw_and_update(LiVESWidget *widget);
 
 boolean global_recent_manager_add(const char *file_name);
 
-void lives_cursor_unref(LiVESXCursor *);
+boolean lives_cursor_unref(LiVESXCursor *);
 
 boolean lives_widget_context_update(void);
 
 LiVESWidget *lives_menu_add_separator(LiVESMenu *);
 
+void lives_menu_item_set_text(LiVESWidget *menuitem, const char *text, boolean use_mnemonic);
+const char *lives_menu_item_get_text(LiVESWidget *menuitem);
+
 boolean lives_widget_get_fg_color(LiVESWidget *, LiVESWidgetColor *);
 
+boolean lives_widget_set_show_hide_parent(LiVESWidget *);
+
 void lives_window_center(LiVESWindow *);
 
 boolean lives_entry_set_completion_from_list(LiVESEntry *, LiVESList *);
 
-void lives_widget_unparent(LiVESWidget *);
+boolean lives_widget_unparent(LiVESWidget *);
 
 void lives_tooltips_copy(LiVESWidget *dest, LiVESWidget *source);
 
@@ -803,12 +976,11 @@ char *lives_text_view_get_text(LiVESTextView *);
 boolean lives_text_view_set_text(LiVESTextView *, const char *text, int len);
 
 boolean lives_text_buffer_insert_at_end(LiVESTextBuffer *, const char *text);
-boolean lives_text_view_scroll_onscreen(LiVESTextView *);
 
 void lives_general_button_clicked(LiVESButton *, livespointer data_to_free);
 
-void lives_spin_button_configure(LiVESSpinButton *, double value, double lower, double upper,
-                                 double step_increment, double page_increment);
+boolean lives_spin_button_configure(LiVESSpinButton *, double value, double lower, double upper,
+                                    double step_increment, double page_increment);
 
 size_t calc_spin_button_width(double min, double max, int dp);
 
@@ -816,32 +988,37 @@ int get_box_child_index(LiVESBox *, LiVESWidget *child);
 
 boolean lives_container_child_set_shrinkable(LiVESContainer *, LiVESWidget *child, boolean val);
 
+boolean set_submenu_colours(LiVESMenu *, LiVESWidgetColor *colf, LiVESWidgetColor *colb);
+
 boolean label_act_toggle(LiVESWidget *, LiVESXEventButton *, LiVESToggleButton *);
 boolean widget_act_toggle(LiVESWidget *, LiVESToggleButton *);
+boolean widget_inact_toggle(LiVESWidget *, LiVESToggleButton *);
+boolean label_act_lockbutton(LiVESWidget *, LiVESXEventButton *, LiVESButton *);
 
-void toggle_button_toggle(LiVESToggleButton *);
-
-void set_child_colour(LiVESWidget *, boolean set_all);
-void set_child_alt_colour(LiVESWidget *, boolean set_all);
+boolean toggle_button_toggle(LiVESToggleButton *);
 
 void funkify_dialog(LiVESWidget *dialog);
-boolean draw_cool_toggle(LiVESWidget *, lives_painter_t *, livespointer);
+EXPOSE_FN_PROTOTYPE(draw_cool_toggle);
 void lives_cool_toggled(LiVESWidget *tbutton, livespointer);
 
-void unhide_cursor(LiVESXWindow *);
+boolean unhide_cursor(LiVESXWindow *);
 void hide_cursor(LiVESXWindow *);
 
-void get_border_size(LiVESWidget *win, int *bx, int *by);
+boolean get_border_size(LiVESWidget *win, int *bx, int *by);
+boolean lives_window_get_inner_size(LiVESWindow *, int *x, int *y);
 
 LiVESWidget *add_hsep_to_box(LiVESBox *);
 LiVESWidget *add_vsep_to_box(LiVESBox *);
 
 LiVESWidget *add_fill_to_box(LiVESBox *);
+LiVESWidget *add_spring_to_box(LiVESBox *, int min);
 
 LiVESWidget *lives_toolbar_insert_space(LiVESToolbar *);
 LiVESWidget *lives_toolbar_insert_label(LiVESToolbar *, const char *labeltext);
+LiVESWidget *lives_standard_tool_button_new(LiVESToolbar *, GtkWidget *icon_widget, const char *label, const char *tooltips);
+boolean lives_tool_button_set_border_colour(LiVESWidget *button, LiVESWidgetState state, LiVESWidgetColor *);
 
-boolean lives_accel_path_disconnect(LiVESAccelGroup *group, const char *path);
+boolean lives_accel_path_disconnect(LiVESAccelGroup *, const char *path);
 
 boolean lives_widget_get_mod_mask(LiVESWidget *, LiVESXModifierType *modmask);
 
@@ -868,34 +1045,86 @@ typedef enum {
 
 void lives_set_cursor_style(lives_cursor_t cstyle, LiVESWidget *);
 
-typedef enum {
-  LIVES_EXPAND_DEFAULT,
-  LIVES_EXPAND_NONE,
-  LIVES_EXPAND_EXTRA
-} lives_expand_t;
+typedef int lives_expand_t;
+#define LIVES_EXPAND_NONE 0
+#define LIVES_EXPAND_DEFAULT_HEIGHT 1
+#define LIVES_EXPAND_DEFAULT_WIDTH 2
+#define LIVES_EXPAND_DEFAULT (LIVES_EXPAND_DEFAULT_HEIGHT | LIVES_EXPAND_DEFAULT_WIDTH)
+#define LIVES_EXPAND_EXTRA_HEIGHT 4
+#define LIVES_EXPAND_EXTRA_WIDTH 8
+#define LIVES_EXPAND_EXTRA (LIVES_EXPAND_EXTRA_HEIGHT | LIVES_EXPAND_EXTRA_WIDTH)
+
+#define LIVES_SHOULD_EXPAND (widget_opts.expand != LIVES_EXPAND_NONE)
+#define LIVES_SHOULD_EXPAND_DEFAULT (widget_opts.expand == LIVES_EXPAND_DEFAULT)
+#define LIVES_SHOULD_EXPAND_EXTRA (widget_opts.expand == LIVES_EXPAND_EXTRA)
+
+#define LIVES_SHOULD_EXPAND_WIDTH (widget_opts.expand & (LIVES_EXPAND_DEFAULT_WIDTH | LIVES_EXPAND_EXTRA_WIDTH))
+#define LIVES_SHOULD_EXPAND_HEIGHT (widget_opts.expand & (LIVES_EXPAND_DEFAULT_HEIGHT | LIVES_EXPAND_EXTRA_HEIGHT))
+
+#define LIVES_SHOULD_EXPAND_EXTRA_WIDTH (widget_opts.expand & LIVES_EXPAND_EXTRA_WIDTH)
+#define LIVES_SHOULD_EXPAND_EXTRA_HEIGHT (widget_opts.expand & LIVES_EXPAND_EXTRA_HEIGHT)
+
+#define LIVES_SHOULD_EXPAND_DEFAULT_WIDTH (LIVES_SHOULD_EXPAND_WIDTH && !LIVES_SHOULD_EXPAND_EXTRA_WIDTH)
+#define LIVES_SHOULD_EXPAND_DEFAULT_HEIGHT (LIVES_SHOULD_EXPAND_HEIGHT && !LIVES_SHOULD_EXPAND_EXTRA_HEIGHT)
+
+
+#define LIVES_SHOULD_EXPAND_DEFAULT_FOR(box) ((LIVES_IS_HBOX(box) && LIVES_SHOULD_EXPAND_DEFAULT_WIDTH) || (LIVES_IS_VBOX(box) && LIVES_EXPAND_DEFAULT_HEIGHT))
+
+#define LIVES_SHOULD_EXPAND_EXTRA_FOR(box) ((LIVES_IS_HBOX(box) && LIVES_SHOULD_EXPAND_EXTRA_WIDTH) || (LIVES_IS_VBOX(box) && LIVES_SHOULD_EXPAND_EXTRA_HEIGHT))
+
+#define LIVES_SHOULD_EXPAND_FOR(box) (LIVES_SHOULD_EXPAND_DEFAULT_FOR(box) || LIVES_SHOULD_EXPAND_EXTRA_FOR(box))
+
+LiVESList *get_textsizes_list(void);
+const char *lives_textsize_to_string(int val);
+
+// custom stock images
+#define LIVES_LIVES_STOCK_AUDIO "lives-audio"
+#define LIVES_LIVES_STOCK_PLAY_SEL "lives-playsel"
+#define LIVES_LIVES_STOCK_FULLSCREEN "lives-fullscreen"
+#define LIVES_LIVES_STOCK_SEPWIN "lives-sepwin"
+#define LIVES_LIVES_STOCK_VOLUME_MUTE "lives-volume_mute"
+#define LIVES_LIVES_STOCK_LOOP "lives-loop"
+#define LIVES_LIVES_STOCK_ZOOM_IN "lives-zoom-in"
+#define LIVES_LIVES_STOCK_ZOOM_OUT "lives-zoom-out"
+#define LIVES_LIVES_STOCK_PREF_GUI "lives-pref_gui"
+#define LIVES_LIVES_STOCK_PREF_DECODING "lives-pref_decoding"
+#define LIVES_LIVES_STOCK_PREF_DIRECTORY "lives-pref_directory"
+#define LIVES_LIVES_STOCK_PREF_EFFECTS "lives-pref_effects"
+#define LIVES_LIVES_STOCK_PREF_ENCODING "lives-pref_encoding"
+#define LIVES_LIVES_STOCK_PREF_JACK "lives-pref_jack"
+#define LIVES_LIVES_STOCK_PREF_MIDI "lives-pref_midi"
+#define LIVES_LIVES_STOCK_PREF_MISC "lives-pref_misc"
+#define LIVES_LIVES_STOCK_PREF_MULTITRACK "lives-pref_multitrack"
+#define LIVES_LIVES_STOCK_PREF_NET "lives-pref_net"
+#define LIVES_LIVES_STOCK_PREF_PLAYBACK "lives-pref_playback"
+#define LIVES_LIVES_STOCK_PREF_RECORD "lives-pref_record"
+#define LIVES_LIVES_STOCK_PREF_THEMES "lives-pref_themes"
+#define LIVES_LIVES_STOCK_PREF_WARNING "lives-pref_warning"
 
 typedef struct {
-  boolean no_gui; // show nothing !
-  boolean swap_label; // swap label/widget position
-  boolean pack_end;
-  boolean line_wrap; // line wrapping for labels
-  boolean mnemonic_label; // if underscore in label text should be mnemonic accelerator
-  boolean non_modal; // non-modal for dialogs
-  lives_expand_t expand; // whether spin,check,radio buttons should expand
-  boolean apply_theme; // whether to apply theming to widget
-  double scale; // scale factor for all sizes
-  int packing_width; // default should be W_PACKING_WIDTH
-  int packing_height; // default should be W_PACKING_HEIGHT
-  int border_width; // default should be W_BORDER_WIDTH
-  int filler_len; // length of extra "fill" between widgets
-  LiVESWidget *last_label; // label widget of last standard widget (spin,radio,check,entry,combo) [readonly]
-  LiVESJustification justify; // justify for labels
-  LiVESJustification default_justify;
-  char **image_filter; /// NULL or NULL terminated list of image extensions which can be loaded
-  char *title_prefix; // Text which is prepended to window titles, etc.
-  int monitor;
-  LiVESXScreen *screen;
-  boolean show_button_images;
+  boolean no_gui; ///< show nothing !
+  boolean swap_label; ///< swap label/widget position
+  boolean pack_end; ///< pack widget at end or start
+  boolean line_wrap; ///< line wrapping for labels
+  boolean mnemonic_label; ///< if underscore in label text should be mnemonic accelerator
+  boolean non_modal; ///< non-modal for dialogs
+  lives_expand_t expand; ///< how much space to apply between widgets
+  boolean apply_theme; ///< whether to apply theming to widget
+  double scale; ///< scale factor for all sizes
+  int packing_width; ///< horizontal pixels between widgets
+  int packing_height; ///< vertical pixels between widgets
+  int border_width; ///< border width in pixels
+  int filler_len; ///< length of extra "fill" between widgets
+  LiVESWidget *last_label; ///< label widget of last standard widget (spin,radio,check,entry,combo) [readonly]
+  LiVESWindow *transient; ///< transient window for dialogs, if NULL then use the default
+  LiVESJustification justify; ///< justify for labels
+  LiVESJustification default_justify; ///< default value
+  const char *font_size; ///<
+  char **image_filter; ///</ NULL or NULL terminated list of image extensions which can be loaded
+  char *title_prefix; ///< Text which is prepended to window titles, etc.
+  int monitor; ///< monitor we are displaying on
+  LiVESXScreen *screen; ///< screen we are displaying on
+  boolean show_button_images; ///< whether to show small images in buttons or not
 } widget_opts_t;
 
 widget_opts_t widget_opts;
@@ -903,27 +1132,29 @@ widget_opts_t widget_opts;
 #ifdef NEED_DEF_WIDGET_OPTS
 
 const widget_opts_t def_widget_opts = {
-  FALSE, // no_gui
-  FALSE, // swap_label
-  FALSE, //pack_end
-  FALSE, // line_wrap
-  TRUE, // mnemonic_label
-  FALSE, // non_modal
-  LIVES_EXPAND_DEFAULT, // default expand
-  FALSE, // no themeing
-  1.0, // default scale
-  W_PACKING_WIDTH, // def packing width
-  W_PACKING_HEIGHT, // def packing height
-  W_BORDER_WIDTH, // def border width
-  W_FILL_LENGTH, // def fill width (in pixels)
-  NULL, // last_label
-  LIVES_JUSTIFY_LEFT, // justify
-  LIVES_JUSTIFY_LEFT, // default justify
-  NULL, // image_filter
-  "", // title_prefix
-  0, // monitor
-  NULL, // screen
-  FALSE // show button images
+  FALSE, ///< no_gui
+  FALSE, ///< swap_label
+  FALSE, ///<pack_end
+  FALSE, ///< line_wrap
+  TRUE, ///< mnemonic_label
+  FALSE, ///< non_modal
+  LIVES_EXPAND_DEFAULT, ///< default expand
+  FALSE, ///< no themeing
+  1.0, ///< default scale
+  W_PACKING_WIDTH, ///< def packing width
+  W_PACKING_HEIGHT, ///< def packing height
+  W_BORDER_WIDTH, ///< def border width
+  W_FILL_LENGTH, ///< def fill width (in pixels)
+  NULL, ///< last_label
+  NULL, ///< transient window
+  LIVES_JUSTIFY_LEFT, ///< justify
+  LIVES_JUSTIFY_LEFT, ///< default justify
+  LIVES_FONT_SIZE_MEDIUM, ///< default font size
+  NULL, ///< image_filter
+  "", ///< title_prefix
+  0, ///< monitor
+  NULL, ///< screen
+  FALSE ///< show button images
 };
 
 #else
